
/*
 * File CASCMode.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __CASCMode__
#define __CASCMode__

#include "Forwards.hpp"

#include "Lib/Portability.hpp"
#include "Lib/ScopedPtr.hpp"
#include "Lib/Set.hpp"
#include "Lib/Stack.hpp"

#include "Lib/VString.hpp"

#include "Shell/Property.hpp"

namespace CASC
{

using namespace std;
using namespace Lib;
using namespace Shell;

class CASCMode {
public:
  virtual ~CASCMode() {}
  static bool perform(int argc,char* argv []);

  typedef Stack<vstring> Schedule;
  static void getSchedules(Property& prop, Schedule& quick, Schedule& fallback);
  static void getSchedulesSat(Property& prop, Schedule& quick, Schedule& fallback);
  static void getSchedulesEPR(Property& prop, Schedule& quick, Schedule& fallback);
  static void getSchedulesTheory(Property& prop, Schedule& quick, Schedule& fallback);
  static unsigned getSliceTime(vstring sliceCode,vstring& chopped);
  static void makeSat() {_sat=true;}
  static void makeEPR() {_epr=true;}
  static void makeTheory() {_theory=true;}
protected:
 
  virtual bool runSlice(Options& opt) = 0;

  void handleSIGINT() __attribute__((noreturn));

 
  Property* _property;
 
  static bool _sat;
 
  static bool _epr;
 
  static bool _theory;

private:
  typedef Set<vstring> StrategySet;
  bool perform();
  bool runSchedule(Schedule&,unsigned ds,StrategySet& remember,bool fallback);
  bool runSlice(vstring sliceCode, unsigned ds);
};

}

#endif 
