
/*
 * File CMZRMode.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __CMZRMode__
#define __CMZRMode__

#include <utility>

#include "Forwards.hpp"

#include "Lib/DHSet.hpp"
#include "Lib/Portability.hpp"
#include "Lib/ScopedPtr.hpp"
#include "Lib/Stack.hpp"

#include "Lib/VString.hpp"

#include "Lib/Sys/SyncPipe.hpp"

#include "Kernel/Problem.hpp"

#include "Shell/Property.hpp"
#include "Shell/SineUtils.hpp"

namespace CASC {

using namespace std;
using namespace Lib;
using namespace Kernel;

#if COMPILER_MSVC

class CMZRMode
{
public:
  static void perform() { USER_ERROR("casc_mzr mode is not supported on Windows"); }
};

#else

class CMZRProblem;

class CMZRMode
{
public:
  CMZRMode();

  static void perform();
private:
  void perform(istream& batchFile);
  void readInput(istream& batchFile);

  void loadIncludes();
  void loadProblems();

  typedef Stack<vstring> StringStack;

  unsigned getSliceTime(vstring sliceCode);
  void getStrategy(Property& property, StringStack& res);

  struct ProblemInfo {
    ProblemInfo(vstring inputFName="",vstring outputFName="")
    : inputFName(inputFName), outputFName(outputFName), specificFormulas(0), property(0),
      solved(false), runningProcessPID(-1) {}

    vstring inputFName;
    vstring outputFName;

    
    UnitList* specificFormulas;
    bool hasConjecture;
    Property* property;
   
    StringStack schedule;

    
    bool solved;
   
    int runningProcessPID;
   
    unsigned processDueTime;
  };


  typedef List<vstring> StringList;



  vstring category;
 
  int problemTimeLimit;

  bool questionAnswering;

  StringList* theoryIncludes;


  ScopedPtr<Problem> baseProblem;

 
  Property* _axiomProperty;

 
  Stack<ProblemInfo> _problems;


  
  bool canAttemptProblem(unsigned idx) const;
  void attemptProblem(unsigned idx);
  void waitForOneFinished();

  void startStrategyRun(unsigned prbIdx, vstring strategy, unsigned timeMs);
  void strategyRunChild(unsigned prbIdx, vstring strategy, unsigned timeMs) NO_RETURN;

  unsigned _parallelProcesses;
  unsigned _availCoreCnt;
  unsigned _unsolvedCnt;

  typedef DHMap<pid_t,unsigned> ProcessMap;
 
  ProcessMap _processProblems;

  friend class CMZRProblem;
};


#endif 

}

#endif 
