
/*
 * File ShortConflictMetaDP.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __ShortConflictMetaDP__
#define __ShortConflictMetaDP__

#include "Forwards.hpp"

#include "Lib/ScopedPtr.hpp"
#include "Lib/Stack.hpp"

#include "SAT/SAT2FO.hpp"
#include "SAT/SATSolver.hpp"

#include "DecisionProcedure.hpp"


namespace DP {

using namespace Lib;
using namespace Kernel;
using namespace SAT;

class ShortConflictMetaDP : public DecisionProcedure {
public:
  CLASS_NAME(ShortConflictMetaDP);
  USE_ALLOCATOR(ShortConflictMetaDP);

 
  ShortConflictMetaDP(DecisionProcedure* inner, SAT2FO& sat2fo, SATSolver& solver)
  : _inner(inner), _sat2fo(sat2fo), _solver(solver) {}

  virtual void addLiterals(LiteralIterator lits, bool onlyEqualites) {
    CALL("ShortConflictMetaDP::addLiterals");
    _inner->addLiterals(lits, onlyEqualites);
  }

  virtual void reset() {
    CALL("ShortConflictMetaDP::reset");
    _inner->reset();
    _unsatCores.reset();
  }

  virtual Status getStatus(bool getMultipleCores);

  void getModel(LiteralStack& model) override {
    _inner->getModel(model);
  }

 
  virtual unsigned getUnsatCoreCount() { return _unsatCores.size(); }
  virtual void getUnsatCore(LiteralStack& res, unsigned coreIndex);


private:
  unsigned getCoreSize(const LiteralStack& core);


  Stack<LiteralStack> _unsatCores;

  ScopedPtr<DecisionProcedure> _inner;
  SAT2FO& _sat2fo;
  SATSolver& _solver;
};

}

#endif 
