
/*
 * File FiniteModelBuilderIncremental.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __FiniteModelBuilderIncremental__
#define __FiniteModelBuilderIncremental__

#include "Forwards.hpp"

#include "Kernel/MainLoop.hpp"
#include "SAT/SATSolver.hpp"
#include "Lib/ScopedPtr.hpp"
#include "SortInference.hpp"

namespace FMB {
using namespace Lib;
using namespace Kernel;
using namespace Inferences;
using namespace Shell;
using namespace SAT;

class FiniteModelBuilderIncremental : public MainLoop {
public:
  CLASS_NAME(FiniteModedlBuilder);
  USE_ALLOCATOR(FiniteModelBuilderIncremental);    
  
  FiniteModelBuilderIncremental(Problem& prb, const Options& opt);

  static Term* getConstant(unsigned i);

protected:
  virtual void init();
  virtual MainLoopResult runImpl();

private:
  static Array<Term*> _modelConstants;
  static unsigned created;
  static unsigned fchecked;

  void onModelFound(unsigned modelSize);

  void addGroundClauses();
  void addNewInstances(unsigned modelSize);
  void addNewFunctionalDefs(unsigned modelSize);
  void addNewSymmetryAxioms(unsigned modelSize,Stack<Term*>& constants, Stack<Term*>& functions);

  void addNewSymmetryAxioms(unsigned modelSize){
    if(_sortInference){
      ASS(_sortedSignature);
      for(unsigned s=0;s<_sortedSignature->sorts;s++){
        
        addNewSymmetryAxioms(modelSize,
                             _sortedSignature->sortedConstants[s],
                             _sortedSignature->sortedFunctions[s]);
      }
    }
    else{
      
      addNewSymmetryAxioms(modelSize,_unsortedConstants,_unsortedFunctions);
    }
  }

  unsigned addNewTotalityDefs(unsigned modelSize);
  
  unsigned getNextSATVar();
  SATLiteral getSATLiteral(Literal* t);

  void createSolver();
  ScopedPtr<SATSolverWithAssumptions> _solver;
  unsigned _maxSatVar;
  DHMap<Literal*,unsigned> _lookup;
  DHMap<unsigned,Literal*> _revLookup;

  void addSATClause(SATClause* cl);
  SATClauseStack _clausesToBeAdded;

  
  ClauseList* _groundClauses;
  ClauseList* _clauses;
  ClauseList* _functionDefinitionClauses;
  Stack<Term*> _totalityFunctions;
  Stack<Term*> _unsortedConstants;
  Stack<Term*> _unsortedFunctions;
  SortedSignature* _sortedSignature;

  unsigned _constantsCount;
  bool _isComplete;
  bool _sortInference;
  unsigned _maxModelSize;
};

}

#endif 
