
/*
 * File ModelPrinter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __ModelPrinter__
#define __ModelPrinter__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/Term.hpp"


namespace InstGen {

using namespace Kernel;
using namespace SAT;

class ModelPrinter {
public:
  ModelPrinter(IGAlgorithm& iga);

  bool tryOutput(ostream& stm);
private:

  struct InstLitComparator;
  struct PredNumComparator;

  bool haveNonDefaultSorts();
  static bool isEprProblem();
  static bool isEquality(Literal* lit);

  void collectTrueLits();
  void getInstances(LiteralStack& trueLits, TermStack& domain, LiteralStack& instanceAcc);
  void generateNewInstances(Literal* base, TermStack& domain, DHSet<Literal*>& instSet, LiteralStack& instAcc);
  void analyzeEqualityAndPopulateDomain();
  void rewriteLits(LiteralStack& lits);

  void outputDomainSpec(ostream& out);
  void outputFunInterpretations(ostream& out);
  void outputPredInterpretations(ostream& out);

  void collectConstants(Literal* lit);

  DHSet<unsigned> _usedConstantSet;
  TermStack _usedConstants;

  Stack<TermList> _domain;


  typedef DHMap<TermList,TermList> EqMap;
 
  EqMap _rewrites;

  LiteralStack _trueLits;
  LiteralStack _trueEqs;

  IGAlgorithm& _iga;
};

}

#endif 
