
/*
 * File Assignment.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */

#if GNUMP
#include "Lib/Metaiterators.hpp"

#include "Assignment.hpp"

namespace Kernel
{

VarIterator Assignment::getAssignedVars() const
{
  CALL("Assignment::getAssignedVars");

  return pvi( getStaticCastIterator<Var>(AssignmentMap::KeyIterator(_data)) );
}

BoundNumber Assignment::evalCoeffs(Constraint::CoeffIterator coeffs) 
{
  CALL("Assignment::evalCoeffs");

  BoundNumber res = BoundNumber::zero();
  while(coeffs.hasNext()) {
    Constraint::Coeff coeff = coeffs.next();
    ASS(isAssigned(coeff.var));
    res+=BoundNumber(coeff.value) * (*this)[coeff.var];
  }
  return res;
}

bool Assignment::isSatisfied(const Constraint* c)
{
  CALL("Assignment::isSatisfied(const Constraint*)");

  BoundNumber lhs = evalCoeffs(c->coeffs());
  BoundNumber rhs(c->freeCoeff());

  switch(c->type()) {
  case CT_EQ:
    return lhs==rhs;
  case CT_GR:
    return lhs>rhs;
  case CT_GREQ:
    return lhs>=rhs;
  default:
    ASSERTION_VIOLATION;
    return false;
  }
}

bool Assignment::isSatisfied(Kernel::ConstraintList* constraints)
{
  CALL("Assignment::isSatisfied(ConstraintList*)");

  ConstraintList::Iterator cit(constraints);
  while(cit.hasNext()) {
    if(!isSatisfied(cit.next())) {
      return false;
    }
  }
  return true;
}


#if VDEBUG
void Assignment::assertSatisfied(const Constraint* c)
{
  CALL("Assignment::assertSatisfied(const Constraint*)");

  BoundNumber lhs = evalCoeffs(c->coeffs());
  BoundNumber rhs(c->freeCoeff());

  switch(c->type()) {
  case CT_EQ:
    ASS_REP2(lhs==rhs, c->toString(), lhs);
    break;
  case CT_GR:
    ASS_REP2(lhs>rhs, c->toString(), lhs);
    break;
  case CT_GREQ:
    ASS_REP2(lhs>=rhs, c->toString(), lhs);
    break;
  }
}

void Assignment::assertSatisfied(ConstraintList* constraints)
{
  CALL("Assignment::assertSatisfied(ConstraintList*)");

  ConstraintList::Iterator cit(constraints);
  while(cit.hasNext()) {
    assertSatisfied(cit.next());
  }
}
#endif

}
#endif 
