
/*
 * File Grounder.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __Grounder__
#define __Grounder__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/ScopedPtr.hpp"
#include "Lib/SmartPtr.hpp"

#include "Kernel/Term.hpp"

namespace Kernel {

using namespace Lib;
using namespace SAT;

class Grounder {
public:
  CLASS_NAME(Grounder);
  USE_ALLOCATOR(Grounder);
  
  Grounder(SATSolver* satSolver) : _satSolver(satSolver) {}
  virtual ~Grounder() { CALL("Grounder::~Grounder"); }

  
  SATLiteral groundLiteral(Literal* lit,bool use_n);
  SATClause* ground(Clause* cl,bool use_n);
  SATClause* groundNonProp(Clause* cl, bool use_n, Literal** normLits=0);
  void groundNonProp(Clause* cl, SATLiteralStack& acc, bool use_n, Literal** normLits=0);

  static void recordInference(Clause* origClause, SATClause* refutation, Clause* resultClause);

  LiteralIterator groundedLits();

protected:
 
  virtual void normalize(unsigned cnt, Literal** lits) = 0;

private:
  SATLiteral groundNormalized(Literal*);

 
  DHMap<Literal*, unsigned> _asgn;
  
 
  SATSolver* _satSolver;
};

class GlobalSubsumptionGrounder : public Grounder {
  struct OrderNormalizingComparator;

  bool _doNormalization;
public:
  CLASS_NAME(GlobalSubsumptionGrounder);
  USE_ALLOCATOR(GlobalSubsumptionGrounder);

  GlobalSubsumptionGrounder(SATSolver* satSolver, bool doNormalization=true) 
          : Grounder(satSolver), _doNormalization(doNormalization) {}
protected:
  virtual void normalize(unsigned cnt, Literal** lits);
};

class IGGrounder : public Grounder {
public:
  CLASS_NAME(IGGrounder);
  USE_ALLOCATOR(IGGrounder);

  IGGrounder(SATSolver* satSolver);
private:
  TermList _tgtTerm;
protected:
  virtual void normalize(unsigned cnt, Literal** lits);
private:
  class CollapsingApplicator;
  Literal* collapseVars(Literal* lit);
};


}

#endif 
