
/*
 * File Ordering.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __Ordering__
#define __Ordering__

#include "Forwards.hpp"

#include "Debug/Assertion.hpp"

#include "Lib/Comparison.hpp"
#include "Lib/SmartPtr.hpp"

#include "Lib/Allocator.hpp"

namespace Kernel {

using namespace Shell;

class Ordering
{
public:
  CLASS_NAME(Ordering);
  USE_ALLOCATOR(Ordering);

 
  enum Result {
    GREATER=1,
    LESS=2,
    GREATER_EQ=3,
    LESS_EQ=4,
    EQUAL=5,
    INCOMPARABLE=6
  };

  Ordering();
  virtual ~Ordering();

 
  virtual Result compare(Literal* l1,Literal* l2) const = 0;
 
  virtual Result compare(TermList t1,TermList t2) const = 0;

  virtual Comparison compareFunctors(unsigned fun1, unsigned fun2) const = 0;

  void removeNonMaximal(LiteralList*& lits) const;

  static Result fromComparison(Comparison c);

  static Result reverse(Result r)
  {
    switch(r) {
    case GREATER:
      return LESS;
    case GREATER_EQ:
      return LESS_EQ;
    case LESS:
      return GREATER;
    case LESS_EQ:
      return GREATER_EQ;
    case EQUAL:
    case INCOMPARABLE:
      return r;
    default:
      ASSERTION_VIOLATION;
    }
  }
  static const char* resultToString(Result r);

  static Ordering* create(Problem& prb, const Options& opt);

  static bool trySetGlobalOrdering(OrderingSP ordering);
  static Ordering* tryGetGlobalOrdering();

  Result getEqualityArgumentOrder(Literal* eq) const;
protected:

  Result compareEqualities(Literal* eq1, Literal* eq2) const;

private:

  enum ArgumentOrderVals {
   
    AO_UNKNOWN=0,
    AO_GREATER=1,
    AO_LESS=2,
    AO_GREATER_EQ=3,
    AO_LESS_EQ=4,
    AO_EQUAL=5,
    AO_INCOMPARABLE=6
  };


  void createEqualityComparator();
  void destroyEqualityComparator();

  class EqCmp;
 
  EqCmp* _eqCmp;

 
  static OrderingSP s_globalOrdering;
}; 

}

#endif
