
/*
 * File SortHelper.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __SortHelper__
#define __SortHelper__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"

#include "Term.hpp"

namespace Kernel {

class SortHelper {
public:
  static unsigned getResultSort(Term* t);
  static unsigned getResultSort(TermList t, DHMap<unsigned,unsigned>& varSorts);
  static unsigned getArgSort(Term* t, unsigned argIndex);

  static bool tryGetResultSort(Term* t, unsigned& result);
  static bool tryGetResultSort(TermList t, unsigned& result);
  static bool getResultSortOrMasterVariable(Term* t, unsigned& resultSort, TermList& resultVar);
  static bool getResultSortOrMasterVariable(TermList t, unsigned& resultSort, TermList& resultVar);

  static unsigned getEqualityArgumentSort(const Literal* lit);

  static bool tryGetVariableSort(unsigned var, Formula* f, unsigned& res);
  static unsigned getVariableSort(TermList var, Term* t);
  static unsigned getTermSort(TermList trm, Literal* lit);

  static void collectVariableSorts(Unit* u, DHMap<unsigned,unsigned>& map);
  static void collectVariableSorts(Term* t, DHMap<unsigned,unsigned>& map);
  static void collectVariableSorts(Formula* f, DHMap<unsigned,unsigned>& map);

  static bool areSortsValid(Clause* cl);
  static bool areImmediateSortsValid(Term* t);

  static BaseType& getType(Term* t);

  static bool areSortsValid(Term* t, DHMap<unsigned,unsigned>& varSorts);
private:
  static bool tryGetVariableSort(TermList var, Term* t, unsigned& result);

};

}

#endif 
