
/*
 * File Unit.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __Unit__
#define __Unit__

#include "Forwards.hpp"

#include "Lib/List.hpp"
#include "Lib/VString.hpp"

namespace Kernel {

using namespace std;
using namespace Lib;

class Unit
{
protected:
  ~Unit() {}
public:
 
  enum Kind {
   
    CLAUSE = 0,
   
    FORMULA = 1
  };

 
  enum InputType {
   
    AXIOM = 0,
   
    ASSUMPTION = 1,
   
    CONJECTURE = 2,
   
    NEGATED_CONJECTURE = 3,
   
    CLAIM = 4,
  };

  static InputType getInputType(UnitList* units);
  static InputType getInputType(InputType t1, InputType t2);

  void destroy();
  vstring toString() const;
  unsigned varCnt();

  vstring inferenceAsString(BDDNode* propPart=0) const;

 
  bool isClause() const
  { return _kind == CLAUSE; }

 
  InputType inputType() const
  { return (InputType)_inputType; }
 
  void setInputType(InputType it)
  { _inputType=it; }

 
  unsigned number() const { return _number; }

 
  Inference* inference() { return _inference; }
 
  const Inference* inference() const { return _inference; }
 
  int adam() const {return _adam;}

 
  inline Color inheritedColor() const
  {
    return static_cast<Color>(_inheritedColor);
  }
  void setInheritedColor(Color color)
  {
    ASS_NEQ(color,COLOR_INVALID);
    _inheritedColor = color;
  } 

  Color getColor();
  Formula* getFormula();
  void collectAtoms(Stack<Literal*>& acc);
  void collectPredicates(Stack<unsigned>& acc);

 
  void incRefCnt();
 
  void decRefCnt();

 
  inline void markIncluded() {_included = 1;}
 
  inline bool included() const {return _included;}

 
  inline bool isFromPreprocessing()
  { return !_firstNonPreprocessingNumber || _number<_firstNonPreprocessingNumber; }


  void assertValid();


  static void onPreprocessingEnd();

protected:
 
  unsigned _number;
 
  unsigned _kind : 1;
 
  unsigned _inputType : 2;
 
  unsigned _inheritedColor : 2;
 
  unsigned _included : 1;
 
  Inference* _inference;
 
  int _adam;

  Unit(Kind kind,Inference* inf,InputType it);

 
  static unsigned _lastNumber;

 
  static unsigned _firstNonPreprocessingNumber;
}; 

std::ostream& operator<< (ostream& out, const Unit& u );

}
#endif
