
/*
 * File V2CIndex.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __V2CIndex__
#define __V2CIndex__
#if GNUMP

#include "Forwards.hpp"

#include "Lib/DArray.hpp"
#include "Lib/Stack.hpp"
#include "Lib/RCPtr.hpp"

#include "Constraint.hpp"

namespace Kernel {

using namespace Lib;

class V2CIndex {
public:
  typedef VirtualIterator<Constraint*> Iterator;

  V2CIndex();

  void insert(const ConstraintRCPtr& c);
  void insert(Constraint* c) { insert(ConstraintRCPtr(c)); }
  void insert(ConstraintRCList* lst);

  Iterator getConsraintsWithBound(const BoundId& b) const;
  Iterator getConsraintsWithComplementary(const BoundId& b) const;

  ConstraintRCStack& getStack(const BoundId& b)
  {
    CALL("V2CIndex::getStack");
    ASS_L(b.var, _varCnt);

    return b.left ? _pos[b.var] : _neg[b.var];
  }

  const ConstraintRCStack& getStack(const BoundId& b) const
  {
    return const_cast<V2CIndex&>(*this).getStack(b);
  }

  size_t getConstraintCnt(const BoundId& b) const
  {
    return getStack(b).size();
  }

 
  size_t getConstraintCnt(Var v) const
  {
    return getConstraintCnt(BoundId(v, true))+getConstraintCnt(BoundId(v, false));
  }

 
  Constraint* getOnlyConstraint(const BoundId& b) const
  {
    CALL("V2CIndex::getOnlyConstraint");

    const ConstraintRCStack& s = getStack(b);
    if(s.size()!=1) {
      return 0;
    }
    return s.top().ptr();
  }

  void reset();
private:

  Var _varCnt;
  DArray<ConstraintRCStack> _pos;
  DArray<ConstraintRCStack> _neg;
};

}
#endif 
#endif 
