
/*
 * File BucketSorter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */



#ifndef __BucketSorter__
#define __BucketSorter__

#include "Stack.hpp"

namespace Lib {

template<typename T, class Evaluator>
class BucketSorter
{
public:
 
  BucketSorter(size_t bucketCnt)
  : _bucketCnt(bucketCnt), _size(0), _minOccupied(bucketCnt)
  {
    CALL("BucketSorter::BucketSorter");

    _buckets=static_cast<Stack<T>*>(ALLOC_KNOWN(_bucketCnt*sizeof(Stack<T>), "BucketSorter"));
    for(unsigned i=0;i<_bucketCnt;i++) {
      new (&_buckets[i]) Stack<T>(8);
    }
  }

 
  ~BucketSorter()
  {
    CALL("BucketSorter::~BucketSorter");

    for(unsigned i=0;i<_bucketCnt;i++) {
      _buckets[i].~Stack<T>();
    }
    DEALLOC_KNOWN(_buckets, _bucketCnt*sizeof(Stack<T>), "BucketSorter");
  }

 
  inline bool isEmpty() { return _size==0; }

 
  void insert(T obj)
  {
    CALL("BucketSorter::insert");

    _size++;
    size_t i=Evaluator::eval(obj);
    _buckets[i].push(obj);
    if(i<_minOccupied) {
      _minOccupied=i;
    }
  }

 
  T pop()
  {
    CALL("BucketSorter::pop");
    ASS_G(_size,0);

    _size--;
    T res=_buckets[_minOccupied].pop();
    if(_buckets[_minOccupied].isEmpty()) {
      if(_size) {
	while(_buckets[++_minOccupied].isEmpty()) {};
      } else {
	_minOccupied=_bucketCnt;
      }
    }
    return res;
  }

private:
 
  size_t _bucketCnt;
 
  Stack<T>* _buckets;

 
  size_t _size;
 
  size_t _minOccupied;
};

};

#endif
