
/*
 * File Comparison.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __VL_COMPARE__
#define __VL_COMPARE__


namespace Lib {

enum Comparison
{
  LESS = -1,
  EQUAL = 0,
  GREATER = 1
};

enum PartialComparison
{
  PC_LESS = -1,
  PC_EQUAL = 0,
  PC_GREATER = 1,
  PC_INCOMPARABLE = 2
};








struct DefaultComparatorTKV
{
  template<typename T>
  inline static Comparison compare(T o1, T o2)
  {
    return o1>o2 ? GREATER : (o1==o2 ? EQUAL : LESS);
  }
};


template<class Comp>
struct ReversedComparator
{
  template<typename T>
  inline static Comparison compare(T o1, T o2)
  {
    return static_cast<Comparison>(-Comp::compare(o1, o2));
  }
};

template<class Comp1, class Comp2>
class CompositeComaprator
{
public:
  CompositeComaprator(Comp1 c1=Comp1(), Comp2 c2=Comp2())
  : _c1(c1), _c2(c2) {}

  template<typename T>
  Comparison compare(T o1, T o2)
  {
    Comparison res1=_c1.compare(o1,o2);
    return (res1==EQUAL) ? _c2.compare(o1,o2) : res1;
  }
private:
  Comp1 _c1;
  Comp2 _c2;
};




















}

#endif
