
/*
 * File Exception.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#include <string.h>

#include "Int.hpp"

#include "Exception.hpp"
#include "VString.hpp"

namespace Lib
{

int Exception::s_exceptionCounter=0;

Exception::Exception (const char* msg, int line)
  : _message((vstring(msg)+": "+Int::toString(line)).c_str())
{ s_exceptionCounter++; }

void Exception::cry (ostream& str)
{
  str << _message << "\n";
} 


void UserErrorException::cry (ostream& str)
{
  str << "User error: " << _message << "\n";
} 

void InvalidOperationException::cry (ostream& str)
{
  str << "Invalid operation: " << _message << "\n";
} 


SystemFailException::SystemFailException(const vstring msg, int err)
: Exception(msg+" error "+Int::toString(err)+": "+strerror(err)), err(err)
{



}
void SystemFailException::cry (ostream& str)
{
  str << "System fail: " << _message << "\n";
} 


void NotImplementedException::cry (ostream& str)
{
  str << "Not implemented at " << file << ":" << line << "\n";
} 


}
