
/*
 * File Int.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __INT__
#define __INT__

#include "Comparison.hpp"
#include "Portability.hpp"
#include "VString.hpp"

using namespace std;

#ifdef _MSC_VER 
#  undef max
#  undef min
#endif

namespace Lib {


class Int
{
 public:
  static vstring toString(int i);
  static vstring toString(unsigned i);
  static vstring toString(unsigned long i);
  static vstring toString(long l);
 
  static vstring toString(float f) { return toString((double)f); }
  static vstring toString(double d);

  static vstring toHexString(size_t i);

	static bool isInteger(const char* str);

	static inline bool isInteger(const vstring& str) { return isInteger(str.c_str()); }

 
  inline static Comparison compare (int i1, int i2)
  { return i1 < i2 ? LESS : i1 == i2 ? EQUAL : GREATER; }
 
  inline static Comparison compare (unsigned i1, unsigned i2)
  { return i1 < i2 ? LESS : i1 == i2 ? EQUAL : GREATER; }
 
  inline static Comparison compare (long i1, long i2)
  { return i1 < i2 ? LESS : i1 == i2 ? EQUAL : GREATER; }
 
  inline static Comparison compare (unsigned long i1, unsigned long i2)
  { return i1 < i2 ? LESS : i1 == i2 ? EQUAL : GREATER; }
 
  inline static Comparison compare (const void* p1, const void* p2)
  { return p1 < p2 ? LESS : p1 == p2 ? EQUAL : GREATER; }
 
  inline static Comparison compare (float f1, float f2)
  { return f1 < f2 ? LESS : f1 == f2 ? EQUAL : GREATER; }
 
  inline static Comparison compare (double f1, double f2)
  { return f1 < f2 ? LESS : f1 == f2 ? EQUAL : GREATER; }
  static bool stringToLong(const char*,long& result);
  static bool stringToUnsignedLong(const char*,unsigned long& result);
  static bool stringToLong(const vstring& str,long& result);
  static bool stringToInt(const vstring& str,int& result);
  static bool stringToInt(const char* str,int& result);
  static bool stringToUnsignedInt(const char* str,unsigned& result);
  static bool stringToUnsignedInt(const vstring& str,unsigned& result);
  static bool stringToDouble(const char*,double& result);
  static bool stringToDouble(const vstring& str,double& result) { return stringToDouble(str.c_str(), result); }
  static bool stringToFloat(const char*,float& result);
  static bool stringToUnsigned64(const vstring& str,long long unsigned& result);
  static bool stringToUnsigned64(const char* str,long long unsigned& result);
  static int sign(int i);
  static int sign(long i);
  static int sign(double);
  static int max(int i,int j);
  static int min(int i,int j);
  static int gcd(int i,int j);
  
  static bool safeUnaryMinus(int num, int& res);
  static bool safePlus(int arg1, int arg2, int& res);
  static bool safeMinus(int num, int sub, int& res);
  static bool safeMultiply(int arg1, int arg2, int& res);
};



inline
int Int::sign(int i)
{
  return i < 0 ? -1 :
         i == 0 ? 0 :
         1;
}


inline
int Int::sign(long l)
{
  return l < 0 ? -1 :
         l == 0 ? 0 :
         1;
}


inline
int Int::sign(double d)
{
  return d < 0 ? -1 :
         d == 0 ? 0 :
         1;
}

inline
int Int::max (int i,int j)
{
  return i >= j ? i : j;
}

inline
int Int::min (int i,int j)
{
  return i <= j ? i : j;
}


}

#endif  

