
/*
 * File IntUnionFind.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __IntUnionFind__
#define __IntUnionFind__

#include "Reflection.hpp"
#include "Stack.hpp"

namespace Lib {

class IntUnionFind {
public:
  IntUnionFind(int cnt);
  ~IntUnionFind();
  bool doUnion(int c1, int c2);
  int root(int c) const;
  void reset();
  void evalComponents();
  int getComponentCount();
private:

  int _cnt;

  bool _modified;

 
  mutable int* _parents;

 
  int* _data;
  Stack<int> _components;

public:
  class ComponentIterator;
  class ElementIterator
  {
  public:
    DECL_ELEMENT_TYPE(int);

    bool hasNext() { return _next!=-1; }
    int next()
    {
      CALL("IntUnionFind::ElementIterator::next");
      ASS_NEQ(_next,-1);

      int res=_next;
      _next=_data[_next];
      return res;
    }
  private:
    ElementIterator(int first, const int* data)
    : _next(first), _data(data) {}

    int _next;
    const int* _data;

    friend class ComponentIterator;
  };

 
  class ComponentIterator
  {
  public:
    DECL_ELEMENT_TYPE(ElementIterator);
   
    ComponentIterator(const IntUnionFind& obj)
    : _cit(obj._components), _data(obj._data)
    {
      CALL("IntUnionFind::ComponentIterator::ComponentIterator");
      ASS(!obj._modified); 
    }

    bool hasNext() { return _cit.hasNext(); }
    ElementIterator next() { return ElementIterator(_cit.next(), _data); }
  private:
    Stack<int>::ConstIterator _cit;
    const int* _data;
  };
};

}

#endif
