
/*
 * File IntegerSet.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#include <new>

#include "Debug/Assertion.hpp"
#include "Allocator.hpp"
#include "IntegerSet.hpp"

using namespace Lib;

bool IntegerSet::member (int n) const
{
  ASS(n >= 0);
  int index = n/BITS_PER_INT;
  if (index >= _words) {
    return false;
  }
  return _set[index] & (1u << (n % BITS_PER_INT));
} 


void IntegerSet::remove (int n)
{
  ASS(n >= 0);

  int index = n/BITS_PER_INT;
  if (index >= _words) {
    return;
  }
  _set[index] &= ~(1u << (n % BITS_PER_INT));
} 


void IntegerSet::insert (int n)
{
  ASS(n >= 0);

  int index = n/BITS_PER_INT;
  if (index >= _words) { 
    int words = _words*2;
    if (words < index+1) {
      words = index+1;
    }
    void* mem = ALLOC_KNOWN(sizeof(unsigned)*words,"IntegerSet");
    unsigned int* set = array_new<unsigned>(mem, words);
    for (int i = _words-1;i >= 0;i--) {
      set[i] = _set[i];
    }
    for (int j = _words;j < words;j++) {
      set[j] = 0u;
    }
    if (_set) {
      array_delete(_set, _words);
      DEALLOC_KNOWN(_set,sizeof(unsigned)*_words,"IntegerSet");
    }
    _set = set;
    _words = words;
  }
  _set[index] |= (1u << (n % BITS_PER_INT));
} 

IntegerSet::~IntegerSet()
{
  if (_set) {
    array_delete(_set, _words);
    DEALLOC_KNOWN(_set,sizeof(unsigned)*_words,"IntegerSet");
  }
} 
