
/*
 * File InverseLookup.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */



#ifndef __InverseLookup__
#define __InverseLookup__

#include "Hash.hpp"
#include "DHMap.hpp"

#include "Lib/Allocator.hpp"

namespace Lib {

template<typename T>
class InverseLookup
{
private:
  
  InverseLookup(const InverseLookup&);
  InverseLookup& operator=(const InverseLookup&);
public:
  CLASS_NAME(InverseLookup<T>);
  USE_ALLOCATOR(InverseLookup<T>);

  template<typename Arr>
  InverseLookup(Arr arr, size_t size)
  {
    for(size_t i=0;i<size;i++) {
      ALWAYS(_data.insert(arr[i],i));
    }
  }

  template<typename Arr>
  void update(Arr arr)
  {
    size_t size=_data.size();
    for(size_t i=0;i<size;i++) {
      NEVER(_data.set(arr[i],i));
    }
  }

  size_t get(T* val)
  {
    return _data.get(val);
  }

private:
  DHMap<T*,size_t,PtrIdentityHash> _data;
};

};

#endif
