
/*
 * File MaybeBool.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __MaybeBool__
#define __MaybeBool__

#include "Forwards.hpp"

#include "Debug/Assertion.hpp"
#include "Debug/Tracer.hpp"


namespace Lib {

class MaybeBool
{
public:
  enum Value {
    FALSE = 0,
    TRUE = 1,
    UNKNOWN = 2
  };

  MaybeBool() : _value(UNKNOWN) {}
  MaybeBool(bool val) : _value(val ? TRUE : FALSE) {}
  MaybeBool(Value val) : _value(val) {}

  bool known() const { return _value!=UNKNOWN; }
  bool isTrue() const { return _value==TRUE; }
  bool isFalse() const { return _value==FALSE; }

  bool operator==(const MaybeBool& o) const { return _value==o._value; }
  bool operator==(MaybeBool::Value o) const { return _value==o; }

  bool value() const {
    CALL("MaybeBool::value");
    ASS(known());
    return _value==TRUE;
  }

  void makeUnknown() { _value = UNKNOWN; }
  void mightBecameFalse() { if(isTrue()) { makeUnknown(); } }
  void mightBecameTrue() { if(isFalse()) { makeUnknown(); } }

private:
  Value _value;
};

inline
std::ostream& operator<< (std::ostream& out, const MaybeBool& u )
{
  return out << (u.isFalse() ? "0" : u.isTrue() ? "1" : "UNKNOWN");
}


}

#endif 
