
/*
 * File Metaarrays.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */



#ifndef __Metaarrays__
#define __Metaarrays__

#include "Forwards.hpp"

#include "VirtualIterator.hpp"

namespace Lib {

template<class Arr>
class ReferencedArrayWrapper
{
public:
  DECL_ELEMENT_TYPE( ELEMENT_TYPE(Arr) );
  explicit ReferencedArrayWrapper(Arr& a) : _a(a) {}
  
  template<typename Idx>
  OWN_ELEMENT_TYPE operator[] (Idx index) { return _a[index]; }
  
  size_t size() {return _a.size(); }
private:
  Arr& _a;
};

template<class Arr>
ReferencedArrayWrapper<Arr> wrapReferencedArray(Arr& a)
{
  return ReferencedArrayWrapper<Arr>(a);
}


template<class Arr, typename Functor>
class MappingArray
{
public:
  DECL_ELEMENT_TYPE(RETURN_TYPE(Functor));
  explicit MappingArray(Arr arr, Functor func)
  : _arr(arr), _func(func) {}

  template<typename Idx>
  RETURN_TYPE(Functor) operator[] (Idx index) { return _func(_arr[index]); }
  size_t size() {return _arr.size(); }
private:
  Arr _arr;
  Functor _func;
};

template<class Arr, typename Functor>
MappingArray<Arr,Functor> getMappingArray(Arr arr, Functor func)
{
  return MappingArray<Arr,Functor>(arr, func);
}


};

#endif
