
/*
 * File MultiCounter.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#include "Debug/Tracer.hpp"

#include "MultiCounter.hpp"
#include "Int.hpp"
#include "Allocator.hpp"
#include "XML.hpp"
#include "Exception.hpp"

using namespace Lib;

void MultiCounter::expandToFit (int v)
{
  CALL("MultiCounter::expandToFit");

  
  int newTop = max(_top*2,v+1);

  
  void* mem = ALLOC_KNOWN(newTop*sizeof(int),"MultiCounter");
  int* newCounts = array_new<int>(mem, newTop);
  
  for (int i = 0;i < _top;i++) {
    newCounts[i] = _counts[i];
  }
  for (int k = _top;k < newTop;k++) {
    newCounts[k] = 0;
  }
  if (_counts) {
    array_delete(_counts,_top);
    DEALLOC_KNOWN(_counts,_top*sizeof(int),"MultiCounter");
  }
  _counts = newCounts;
  _top = newTop;
} 


MultiCounter::~MultiCounter()
{
  if (_counts) {
    array_delete(_counts, _top);
    DEALLOC_KNOWN(_counts,_top*sizeof(int),"MultiCounter");
  }
} 


#if VDEBUG

















#endif
