
/*
 * File MultiCounter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __MultiCounter__
#define __MultiCounter__

#include "Debug/Assertion.hpp"
#include "XML.hpp"

namespace Lib {

class MultiCounter
{
public:
 
  MultiCounter()
    :  _top(0),
       _counts(0)
  {}

  ~MultiCounter();

  void inc(int v)
  {
    if (v >= _top) { 
      expandToFit(v);
    }
    _counts[v]++;
  } 

 
  void dec(int v)
  {
    if (v >= _top) { 
      expandToFit(v);
    }
    _counts[v]--;
  } 

 
  void set(int v, int c)
  {
    if (v >= _top) { 
      expandToFit(v);
    }
    _counts[v] = c;
  } 

 
  int get(int v) const
  {
    return v < _top ? _counts[v] : 0; 
  } 

 
  int unsafeGet(int v) const
  {
    ASS(v < _top);
    return _counts[v]; 
  } 

 
  int lastCounter() const
  { return _top-1; }

#if VDEBUG
  XMLElement toXML() const;
#endif
private:
 
  int _top;
 
  int* _counts;
  void expandToFit(int v);
}; 

}

#endif 


