
/*
 * File Portability.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */
#ifndef __Portability__
#define __Portability__

#include "Debug/Assertion.hpp"




#ifdef _MSC_VER
# define COMPILER_MSVC 1
#else
# define COMPILER_MSVC 0
#endif

#ifndef __APPLE__
# define __APPLE__ 0
#endif

#ifndef __CYGWIN__
# define __CYGWIN__ 0
#endif




#ifdef _LP64
#define ARCH_X64 1
#define ARCH_X86 0
#elif _M_X64

#define ARCH_X64 1
#define ARCH_X86 0
#else
#define ARCH_X64 0
#define ARCH_X86 1
#endif





ASS_STATIC(sizeof(char)==1);





#ifndef __GNUC__
# define  __attribute__(x) 
#endif




#define NO_RETURN __attribute__((noreturn))




#ifdef EFENCE
# define NO_PREFETCH
#endif

#ifdef VALGRIND
# define NO_PREFETCH
#endif

#ifdef NO_PREFETCH
# define PREFETCH(x)
# define PREFETCH_LOC(x)
#else
# ifdef __GNUC__
#  define PREFETCH(x) __builtin_prefetch((x),0,2)
#  define PREFETCH_LOC(x) __builtin_prefetch((x),0,0)
# else
#  define PREFETCH(x)
#  define PREFETCH_LOC(x)
# endif
#endif





#if COMPILER_MSVC

#define int32_t __int32
#define pid_t __int32

#endif





#if COMPILER_MSVC


#define HASH_LITTLE_ENDIAN 1
#define HASH_BIG_ENDIAN 0

#else

#include <sys/param.h>  
#ifdef linux
# include <endian.h>    
#endif

#if (defined(__BYTE_ORDER) && defined(__LITTLE_ENDIAN) && \
     __BYTE_ORDER == __LITTLE_ENDIAN) || \
    (defined(i386) || defined(__i386__) || defined(__i486__) || \
     defined(__i586__) || defined(__i686__) || defined(vax) || defined(MIPSEL))
# define HASH_LITTLE_ENDIAN 1
# define HASH_BIG_ENDIAN 0
#elif (defined(__BYTE_ORDER) && defined(__BIG_ENDIAN) && \
       __BYTE_ORDER == __BIG_ENDIAN) || \
      (defined(sparc) || defined(POWERPC) || defined(mc68000) || defined(sel))
# define HASH_LITTLE_ENDIAN 0
# define HASH_BIG_ENDIAN 1
#else
# define HASH_LITTLE_ENDIAN 1
# define HASH_BIG_ENDIAN 0
#endif

#endif

#endif
