
/*
 * File RCPtr.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __RCPtr__
#define __RCPtr__

#include "Forwards.hpp"

#include "Debug/Assertion.hpp"
#include "Debug/Tracer.hpp"

#include "Metaiterators.hpp"

namespace Lib {

template<class T>
class RCPtr {
public:
  inline
  RCPtr() : _obj(0) {}
 
  inline
  explicit RCPtr(T* obj) : _obj(obj)
  {
    if(obj) {
      _obj->incRefCnt();
    }
  }
  inline
  RCPtr(const RCPtr& ptr) : _obj(ptr._obj)
  {
    if(_obj) {
      _obj->incRefCnt();
    }
  }
  inline
  ~RCPtr()
  {
    if(_obj) {
      _obj->decRefCnt();
    }
  }
  RCPtr& operator=(T* newObj)
  {
    CALL("RCPtr::operator=(T*)");

    T* oldObj=_obj;
    _obj=newObj;

    if(_obj) {
      _obj->incRefCnt();
    }

    if(oldObj) {
      oldObj->decRefCnt();
    }
    return *this;
  }
  RCPtr& operator=(const RCPtr& ptr)
  {
    CALL("RCPtr::operator=(const RCPtr&)");

    return (*this)=ptr.ptr();
  }
  bool operator==(const RCPtr& o) const { return ptr()==o.ptr(); }
  bool operator!=(const RCPtr& o) const { return !((*this)==o); }

  inline
  operator bool() const { return _obj; }

  inline
  T* operator->() const { ASS(_obj); return _obj; }
  inline
  T& operator*() const { ASS(_obj); return *_obj; }

  inline
  T* ptr() const { return _obj; }

  inline
  bool isEmpty() const { return !_obj; }

  template<class Target>
  inline
  Target* pcast() const { return static_cast<Target*>(_obj); }

  static List<T*>* unRCList(List<RCPtr>* l)
  {
    CALL("RCPtr::unRCList");

    List<T*>* res = 0;
    typename List<RCPtr>::Iterator it(l);
    while(it.hasNext()) {
      List<T*>::push(it.next().ptr(), res);
    }
    return res;
  }
  static List<RCPtr>* enRCList(List<T*>* l)
  {
    CALL("RCPtr::enRCList");

    List<RCPtr>* res = 0;
    List<RCPtr>::pushFromIterator(getStaticCastIterator<RCPtr>(List<T*>::Iterator(l)), res);
    return res;
  }

  struct UnRCFunctor
  {
    DECL_RETURN_TYPE(T*);
    OWN_RETURN_TYPE operator()(const RCPtr& p) const {
      return p.ptr();
    }
  };
private:
  T* _obj;
};

}

#endif 
