
/*
 * File RatioKeeper.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __RatioKeeper__
#define __RatioKeeper__

#include "Forwards.hpp"

#include "Debug/Tracer.hpp"

namespace Lib {

class RatioKeeper {
public:
  RatioKeeper(int firstRatio, int secondRatio, unsigned buffer=0)
  : _firstRatio(firstRatio), _secondRatio(secondRatio), _buffer(buffer), _lastWasFirst(true), _balance(0) {}

  bool shouldDoFirst() const
  {
    return balanceFitsBuffer() ? _lastWasFirst : _balance>=0;
  }
  bool shouldDoSecond() const
  {
    return balanceFitsBuffer() ? !_lastWasFirst : _balance<0;
  }

  void doFirst(int cost=1)
  {
    CALL("RatioKeeper::doFirst");
    ASS(shouldDoFirst());
    _balance -= cost*_secondRatio;
    _lastWasFirst = true;
  }

  void doSecond(int cost=1)
  {
    CALL("RatioKeeper::doSecond");
    ASS(shouldDoSecond());
    _balance += cost*_firstRatio;
    _lastWasFirst = false;
  }

  void alwaysDoFirst()
  {
    _firstRatio = _secondRatio = 0;
    _balance = 1;
    _lastWasFirst = true;
  }
  void alwaysDoSecond()
  {
    _firstRatio = _secondRatio = 0;
    _balance = -1;
    _lastWasFirst = false;
  }
private:
  bool balanceFitsBuffer() const { return _balance<_buffer && _balance>-_buffer; }

  int _firstRatio;
  int _secondRatio;
  int _buffer;

  bool _lastWasFirst;
  int _balance;
};

}

#endif 
