
/*
 * File STLAllocator.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */



#ifndef __STLAllocator__
#define __STLAllocator__

#include <limits>

#include "Lib/Allocator.hpp"

namespace Lib {

template<typename T>
class STLAllocator {
public : 
    
    typedef T value_type;
    typedef value_type* pointer;
    typedef const value_type* const_pointer;
    typedef value_type& reference;
    typedef const value_type& const_reference;
    typedef std::size_t size_type;
    typedef std::ptrdiff_t difference_type;

public : 
    
    template<typename U>
    struct rebind {
        typedef STLAllocator<U> other;
    };

public : 
    inline explicit STLAllocator() {}
    inline ~STLAllocator() {}
    inline explicit STLAllocator(STLAllocator const&) {}
    template<typename U>
    inline STLAllocator(STLAllocator<U> const&) {}

    
    inline pointer address(reference r) { return &r; }
    inline const_pointer address(const_reference r) { return &r; }

    
    inline pointer allocate(size_type cnt, 
       typename std::allocator<void>::const_pointer = 0) { 
      return reinterpret_cast<pointer>(ALLOC_UNKNOWN(cnt*sizeof(T),"STLAllocator<T>"));           
    }
    inline void deallocate(pointer p, size_type) { 
        DEALLOC_UNKNOWN(p,"STLAllocator<T>");
    }

    
    inline size_type max_size() const { 
        return std::numeric_limits<size_type>::max() / sizeof(T);
 }

    
    inline void construct(pointer p, const T& t) { new(p) T(t); }
    inline void destroy(pointer p) { p->~T(); }

    inline bool operator==(STLAllocator const&) const { return true; }
    inline bool operator!=(STLAllocator const& a) const { return !operator==(a); }
};    

extern const STLAllocator<void> global_stl_allocator;

} 

#endif 
