
/*
 * File System.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __System__
#define __System__

#include "Forwards.hpp"

#include "Array.hpp"
#include "List.hpp"
#include "Stack.hpp"
#include "Portability.hpp"
#include "VString.hpp"

#define VAMP_RESULT_STATUS_SUCCESS 0
#define VAMP_RESULT_STATUS_UNKNOWN 1
#define VAMP_RESULT_STATUS_OTHER_SIGNAL 2
#define VAMP_RESULT_STATUS_SIGINT 3
#define VAMP_RESULT_STATUS_UNHANDLED_EXCEPTION 4


bool outputAllowed();
bool inSpiderMode();
void reportSpiderFail();
void reportSpiderStatus(char status);

namespace Lib {

using namespace std;

typedef void (*SignalHandler)(int);

class System {
public:

  static void setSignalHandlers();
  static vstring extractFileNameFromPath(vstring str);
  static bool extractDirNameFromPath(vstring path, vstring& dir);

  static vstring guessExecutableDirectory();
  static vstring guessExecutableName();

  static void ignoreSIGINT() { s_shouldIgnoreSIGINT=true; }
  static void heedSIGINT() { s_shouldIgnoreSIGINT=false; }
  static bool shouldIgnoreSIGINT() { return s_shouldIgnoreSIGINT; }

  static void addInitializationHandler(VoidFunc proc, unsigned priority=0);
  static void onInitialization();

  static void addTerminationHandler(VoidFunc proc, unsigned priority=0);
  static void onTermination();
  static void terminateImmediately(int resultStatus) __attribute__((noreturn));

  static void registerForSIGHUPOnParentDeath();

  static void readCmdArgs(int argc, char* argv[], StringStack& res);

 
  static void readDir(vstring dirName, Stack<vstring>& filenames);

 
  static void registerArgv0(const char* argv0) { s_argv0 = argv0; }

 
  static long long getSystemMemory();

 
  static unsigned getNumberOfCores();

  static bool fileExists(vstring fname);

  static pid_t getPID();

  static int executeCommand(vstring command, vstring input, Stack<vstring>& outputLines);

private:

  static ZIArray<List<VoidFunc>*>& initializationHandlersArray();

 
  static bool s_initialized;

 
  static ZIArray<List<VoidFunc>*> s_terminationHandlers;

  static bool s_shouldIgnoreSIGINT;

  static const char* s_argv0;
};

}

#endif
