
/*
 * File XML.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __XML__
#define __XML__

#include <iostream>

#include "VString.hpp"

using namespace std;

namespace Lib {

class XMLAttribute {
public:
 
  enum Type {
    TEXT = 0u,
    LONG = 1u,
    INTEGER = 2u,
    DOUBLE = 3u,
    UNSIGNED = 4u
  };

  class Data;

 
  XMLAttribute() : _data(0) {}
  XMLAttribute(const XMLAttribute& attr);
  XMLAttribute(const vstring& name,const vstring& value);
  XMLAttribute(const vstring& name,int value);
  XMLAttribute(const vstring& name,long value);
  XMLAttribute(const vstring& name,double value);
  XMLAttribute(const vstring& name,unsigned value);
  ~XMLAttribute();
  void operator= (const XMLAttribute& rhs);
  void setNext(const XMLAttribute&);
  void find(const vstring& name,XMLAttribute&) const;
 
  bool isNull() const { return _data == 0; }
  const vstring& name() const;

  
  Type tag () const;
  bool operator == (const XMLAttribute& rhs) const;
  bool operator != (const XMLAttribute& rhs) const;

  
  void* operator new(size_t);

  
  void writeList(ostream&) const;

 
  Data* data() const { return _data; }
 private:
  
  Data* _data;
}; 


class XMLElement {
public:
 
  enum Type {
   
    DEEP = 0u,
   
    EMPTY = 1u,
   
    TEXT = 2u,
   
    INTEGER = 3u,
   
    FLOAT = 4u,
   
    LONG = 5u,
   
    DOUBLE = 6u
  };

  class Data;
  class ChildIterator;

 
  XMLElement() : _data(0) {}
 
  bool isNull() const { return _data == 0; }
  XMLElement(const vstring& name);  
  XMLElement(const vstring& name,bool dummy);  
  XMLElement(const vstring& name,const vstring& text);  
  XMLElement(const vstring& name,int content);         
  XMLElement(const vstring& name,long content);        
  XMLElement(const vstring& name,float content);       
  XMLElement(const vstring& name,double content);
  XMLElement(const XMLElement& elem);
  ~XMLElement();
  void operator= (const XMLElement& rhs);

  void addAttribute(const vstring& name,const vstring& value);
  void addAttribute(const vstring& name,int value);
  void addAttribute(const vstring& name,unsigned value);
  void addAttribute(const vstring& name,long value);
  void addAttribute(const vstring& name,double value);
  void addChild(const XMLElement& element);
  const vstring& name() const;
  const vstring* findStringValue(const vstring& name) const;
  void write(ostream&) const;
  void write(ostream&,int indent) const;
  void append(const vstring& file); 
  void copyAttributes(const XMLElement&);

  
  Type tag() const;
  const vstring& text() const;
  bool operator == (const XMLElement& rhs) const;
  bool operator != (const XMLElement& rhs) const;

  
  void* operator new(size_t);

  static void writeString(ostream& str,const vstring& s);

 
  Data* data() const { return _data; }

  void checkDeep() const;
  void checkEmpty() const;
  void checkText() const;
  bool isEmpty() const;

  XMLElement firstChild() const;
 private:
  
  Data* _data;
}; 


class XMLElement::ChildIterator {
public:
  explicit ChildIterator(const XMLElement&);
  bool hasNext() const;
  XMLElement next();
private:
  XMLElement _next;
};


} 

#endif
