
/*
 * File BufferedSolver.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __BufferedSolver__
#define __BufferedSolver__

#include "Forwards.hpp"

#include "Lib/DArray.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/DHSet.hpp"
#include "Lib/ScopedPtr.hpp"
#include "Lib/Stack.hpp"

#include "SATSolver.hpp"

namespace SAT {

using namespace Lib;

class BufferedSolver : public SATSolver {
public:
  CLASS_NAME(BufferedSolver);
  USE_ALLOCATOR(BufferedSolver);

  BufferedSolver(SATSolver* inner);

  virtual SATClause* getRefutation() { return _inner->getRefutation(); }
  virtual void randomizeForNextAssignment(unsigned maxVar) override {
    _inner->randomizeForNextAssignment(maxVar);

    
    
    flushUnadded();
    _lastStatus = _inner->solve();
  }

  virtual void addClause(SATClause* cl) override;
  virtual Status solve(unsigned conflictCountLimit) override;
  virtual VarAssignment getAssignment(unsigned var) override;

  virtual bool isZeroImplied(unsigned var) {
    CALL("BufferedSolver::isZeroImplied");
    ASS_G(var,0); ASS_LE(var,_varCnt);
    
    return (var > _varCntInnerOld) ? false : _inner->isZeroImplied(var);
  }
  virtual void collectZeroImplied(SATLiteralStack& acc) { _inner->collectZeroImplied(acc); }
  virtual SATClause* getZeroImpliedCertificate(unsigned var) { return _inner->getZeroImpliedCertificate(var); }

  virtual void ensureVarCount(unsigned newVarCnt) override { _inner->ensureVarCount(newVarCnt); _varCnt=max(_varCnt,newVarCnt); }
  virtual unsigned newVar() override { 
    CALL("BufferedSolver::newVar");
    
    ALWAYS(_inner->newVar() == ++_varCnt);
    return _varCnt;
  }
  
  virtual void suggestPolarity(unsigned var,unsigned pol) override { _inner->suggestPolarity(var,pol); }
  virtual void recordSource(unsigned var, Literal* lit){
    _inner->recordSource(var,lit);
  }

private:

  
  
  bool checkAndRecordImplied(SATClause* cl);

  
  void flushUnadded();

  SATSolverSCP _inner;


  DHMap<unsigned, bool> _literalBuffer;

 
  SATClauseStack _unadded;
  
  
  unsigned _checkedIdx;

 
  Status _lastStatus;

 
  unsigned _varCnt;
  
 
  unsigned _varCntInnerOld;
};

}

#endif 
