
/*
 * File SATClause.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */



#ifndef __SATClause__
#define __SATClause__

#include <iosfwd>

#include "Forwards.hpp"

#include "Lib/Allocator.hpp"
#include "Lib/InverseLookup.hpp"
#include "Lib/List.hpp"
#include "Lib/Metaiterators.hpp"
#include "Lib/Reflection.hpp"
#include "Lib/VirtualIterator.hpp"
#include "Lib/VString.hpp"

#include "Kernel/InferenceStore.hpp"

#include "SATLiteral.hpp"

namespace SAT {

using namespace std;
using namespace Lib;
using namespace Kernel;

class SATClause
{
public:
  DECL_ELEMENT_TYPE(SATLiteral);

  typedef ArrayishObjectIterator<SATClause> Iterator;

  DECL_ITERATOR_TYPE(Iterator);

  typedef double ActivityType;

 
  SATClause(unsigned length,bool kept=true);
  
  SATInference* inference() const { return _inference; }
  void setInference(SATInference* val);

  void* operator new(size_t,unsigned length);

 
  inline SATLiteral& operator[] (int n)
  { return _literals[n]; }
 
  inline const SATLiteral& operator[] (int n) const
  { return const_cast<const SATLiteral&>(_literals[n]); }

 
  inline unsigned length() const { return _length; }
 
  inline unsigned size() const { return _length; }

  inline bool kept() const { return _kept; }
  inline void makeKept() { _kept=true; }
  inline void setKept(bool kept) { _kept=kept; }

 
  inline SATLiteral* literals() { return _literals; }

 
  inline bool isEmpty() const { return _length == 0; }

  ActivityType& activity() { return _activity; }

  void sort();

  void destroy();

  vstring toString() const;
  vstring toDIMACSString() const;

  bool hasUniqueVariables() const;

 
  struct NamingContext {
    NamingContext() : nextVar(1) {}

    DHMap<Literal*, int> map;
    unsigned nextVar;
  };
  static SATClauseList* fromFOClauses(ClauseIterator clauses);
  static SATClauseList* fromFOClauses(NamingContext& context, ClauseIterator clauses);
  static SATClause* fromFOClause(NamingContext& context, Clause* clause);

  static SATClause* fromStack(SATLiteralStack& stack);

  static SATClause* copy(SATClause* cl);

protected:
  static SATLiteral litToSAT(NamingContext& context, Literal* lit);

  ActivityType _activity;

 
  unsigned _length : 30;

  unsigned _kept : 1;
  unsigned _nonDestroyable : 1;


  SATInference* _inference;


 
  SATLiteral _literals[1];
}; 

std::ostream& operator<< (std::ostream& out, const SAT::SATClause& cl );

};

#endif
