
/*
 * File Z3Interfacing.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __Z3Interfacing__
#define __Z3Interfacing__

#if VZ3

#include "Lib/DHMap.hpp"

#include "SATSolver.hpp"
#include "SATLiteral.hpp"
#include "SATClause.hpp"
#include "SATInference.hpp"
#include "SAT2FO.hpp"

#include "z3++.h"
#include "z3_api.h"

namespace SAT{

class Z3Interfacing : public PrimitiveProofRecordingSATSolver
{
public: 
  CLASS_NAME(Z3Interfacing);
  USE_ALLOCATOR(Z3Interfacing);
  
  Z3Interfacing(const Shell::Options& opts, SAT2FO& s2f, bool generateProofs=false);

 
  virtual void addClauses(SATClauseIterator cit);
  void addClause(SATClause* cl);

  virtual Status solve(unsigned conflictCountLimit) override;
 
  virtual VarAssignment getAssignment(unsigned var);

 
  virtual void randomizeAssignment();

 
  virtual bool isZeroImplied(unsigned var);
 
  virtual void collectZeroImplied(SATLiteralStack& acc);
 
  virtual SATClause* getZeroImpliedCertificate(unsigned var);

  
  virtual void ensureVarCnt(unsigned newVarCnt) {
    CALL("Z3Interfacing::ensureVarCnt");
    _varCnt = max(newVarCnt,_varCnt);
  }

  virtual unsigned newVar() {
    CALL("Z3Interfacing::newVar");
    return ++_varCnt;
  }

  
  virtual void suggestPolarity(unsigned var, unsigned pol){} 
  virtual void forcePolarity(unsigned var, unsigned pol) {}
  
  virtual void addAssumption(SATLiteral lit) {
    CALL("Z3Interfacing::addAssumption");
    NOT_IMPLEMENTED;
  }
  
  virtual void retractAllAssumptions(){} 
  
  virtual bool hasAssumptions() const{ return false; }



  virtual void recordSource(unsigned satlitvar, Literal* lit) {
    
  };
  
private:
  
  unsigned _varCnt;

  
  SAT2FO& sat2fo;

  
  z3::sort getz3sort(unsigned s);

  
  z3::expr to_int(z3::expr e) {
        return z3::expr(e.ctx(), Z3_mk_real2int(e.ctx(), e));
  }
  z3::expr to_real(z3::expr e) {
        return z3::expr(e.ctx(), Z3_mk_int2real(e.ctx(), e));
  }
  z3::expr ceiling(z3::expr e){
        return -to_real(to_int((e)));
  }
  z3::expr is_even(z3::expr e) {
        z3::context& ctx = e.ctx();
        z3::expr two = ctx.int_val(2);
        z3::expr m = z3::expr(ctx, Z3_mk_mod(ctx, e, two));
        return m == 0;
  }

  z3::expr truncate(z3::expr e) {
        return ite(e >= 0, to_int(e), ceiling(e));
  }
public:
  z3::expr getz3expr(Term* trm,bool islit);
private:
  z3::expr getRepresentation(SATLiteral lit);

  Status _status;
  z3::context _context;
  z3::solver _solver;
  z3::model _model;

  bool _showZ3;
};

}

#endif
#endif
