
/*
 * File AWPassiveClauseContainer.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __AWPassiveClauseContainer__
#define __AWPassiveClauseContainer__

#include "Lib/Comparison.hpp"
#include "Kernel/Clause.hpp"
#include "Kernel/ClauseQueue.hpp"
#include "ClauseContainer.hpp"

#include "Lib/Allocator.hpp"

namespace Saturation {

using namespace Kernel;

class AgeQueue
: public ClauseQueue
{
public:
  AgeQueue(const Options& opt) : _opt(opt) {}
protected:

  virtual bool lessThan(Clause*,Clause*);

  friend class AWPassiveClauseContainer;

private:
  const Options& _opt;
};

class WeightQueue
  : public ClauseQueue
{
public:
  WeightQueue(const Options& opt) : _opt(opt) {}
protected:
  virtual bool lessThan(Clause*,Clause*);

  friend class AWPassiveClauseContainer;
private:
  const Options& _opt;
};

class AWPassiveClauseContainer
: public PassiveClauseContainer
{
public:
  CLASS_NAME(AWPassiveClauseContainer);
  USE_ALLOCATOR(AWPassiveClauseContainer);

  AWPassiveClauseContainer(const Options& opt);
  virtual ~AWPassiveClauseContainer();
  void add(Clause* cl);

  void remove(Clause* cl);

  Clause* popSelected();
 
  bool isEmpty() const
  { return _ageQueue.isEmpty() && _weightQueue.isEmpty(); }

  ClauseIterator iterator();

  void updateLimits(long long estReachableCnt);

  virtual unsigned size() const { return _size; }


  static Comparison compareWeight(Clause* cl1, Clause* cl2, const Options& opt);
protected:
  void onLimitsUpdated(LimitsChangeType change);

private:

 
  AgeQueue _ageQueue;
 
  WeightQueue _weightQueue;
 
  int _ageRatio;
 
  int _weightRatio;
 
  int _balance;

  unsigned _size;

  const Options& _opt;
}; 

class AWClauseContainer: public ClauseContainer
{
public:
  AWClauseContainer(const Options& opt);

  void add(Clause* cl);
  bool remove(Clause* cl);

 
  void setAgeWeightRatio(int age,int weight)
  {
    ASS(age >= 0);
    ASS(weight >= 0);
    ASS(age > 0 || weight > 0);

    _ageRatio = age;
    _weightRatio = weight;
  }

  Clause* popSelected();
 
  bool isEmpty() const;

  unsigned size() const { return _size; }

  static Comparison compareWeight(Clause* cl1, Clause* cl2);

private:
 
  AgeQueue _ageQueue;
 
  WeightQueue _weightQueue;
 
  int _ageRatio;
 
  int _weightRatio;
 
  int _balance;

  unsigned _size;
};


};

#endif
