
/*
 * File Otter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */



#ifndef __Otter__
#define __Otter__

#include "Forwards.hpp"

#include "SaturationAlgorithm.hpp"

namespace Saturation {

using namespace Kernel;

class Otter
: public SaturationAlgorithm
{
public:
  CLASS_NAME(Otter);
  USE_ALLOCATOR(Otter);

  Otter(Problem& prb, const Options& opt);

  ClauseContainer* getSimplifyingClauseContainer();

protected:

  
  void onSOSClauseAdded(Clause* cl);

  
  void onActiveRemoved(Clause* cl);

  
  void onPassiveAdded(Clause* cl);
  
  void onPassiveRemoved(Clause* cl);

  
  void onClauseRetained(Clause* cl);



  
  void handleUnsuccessfulActivation(Clause* c);

 
  struct FakeContainer
  : public ClauseContainer
  {
   
    void add(Clause* c)
    { addedEvent.fire(c); }

   
    void remove(Clause* c)
    { removedEvent.fire(c); }
  };

  FakeContainer _simplCont;
};

};

#endif
