
/*
 * File SymElOutput.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __SymElOutput__
#define __SymElOutput__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"

#include "Shell/TPTPPrinter.hpp"

namespace Saturation {

using namespace Lib;
using namespace Kernel;
using namespace Shell;

class SymElOutput {
public:
  CLASS_NAME(SymElOutput);
  USE_ALLOCATOR(SymElOutput);
  
  SymElOutput();

  void init(SaturationAlgorithm* sa);

  void onAllProcessed();
  void onInputClause(Clause* c);
  void onNonRedundantClause(Clause* c);
  void onParenthood(Clause* cl, Clause* parent);


private:

  void onSymbolElimination(Color eliminated, Clause* c, bool nonRedundant=false);

  void outputSymbolElimination(Color eliminated, Clause* c);

  void checkForPreprocessorSymbolElimination(Clause* cl);

 
  unsigned _symElNextClauseNumber;

 
  DHMap<Clause*,Clause*> _symElRewrites;

 
  DHMap<Clause*,Color> _symElColors;


  SaturationAlgorithm* _sa;

  TPTPPrinter _printer;
};

}

#endif 
