
/*
 * File AIGCompressor.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __AIGCompressor__
#define __AIGCompressor__

#include "Forwards.hpp"

#include "Lib/SharedSet.hpp"

#include "Kernel/BDD.hpp"

#include "AIG.hpp"

namespace Kernel {
class InterpretedLiteralEvaluator;
}

namespace Shell {

using namespace Lib;
using namespace Kernel;

class BDDAIG {
public:
  BDDAIG(AIG& aig) : _nextVar(1), _bdd(BDD::instance()), _aig(aig) {};

  void loadBDDAssignmentFromProblem(const Problem& prb);

 
  BDDNode* a2b(AIGRef a);
  AIGRef b2a(BDDNode* b);
  void reset();
private:
  struct A2BConjBuildingTask;
  struct B2ATask;

  BDDNode* bddFromAIGConj(AIGRef node, BDDNode* c1, BDDNode* c2);
  bool attemptLocalA2B(AIGRef a, BDDNode*& res);
  bool attemptLocalA2BOrAddTask(AIGRef a, BDDNode** resTgt, Stack<A2BConjBuildingTask*>& taskStack);

  AIGRef aigFromCompoundBDD(BDDNode* b, AIGRef tAig, AIGRef fAig);
  bool attemptLocalB2A(BDDNode* b, AIGRef& res);
  bool attemptLocalB2AOrAddTask(BDDNode* b, AIGRef* resTgt, Stack<B2ATask*>& taskStack);
  AIGRef naiveB2A(BDDNode* b);


  unsigned _nextVar;

  DHMap<AIGRef, BDDNode*> _a2bCache;
  DHMap<BDDNode*, AIGRef> _b2aCache;
  DHMap<unsigned, AIGRef> _varAtoms;

  BDD* _bdd;
  AIG& _aig;
};

class AIGCompressor {
public:
  typedef AIGTransformer::RefMap RefMap;

  AIGCompressor(AIG& aig, unsigned reqFactorNum=1, unsigned reqFactorDenom=1, unsigned maxBddVarCnt=16);
  ~AIGCompressor();

  AIGRef compress(AIGRef aig);
  AIGRef compressByBDD(AIGRef aig);

  void populateBDDCompressingMap(AIGInsideOutPosIterator& aigIt, AIGTransformer::RefMap& map);

private:
  AIGRef tryCompressAtom(AIGRef atom);


  bool tryCompareAIGGoodness(AIGRef a1, AIGRef a2, Comparison& res);
  bool doHistoryLookUp(AIGRef aig, BDDNode* bdd, AIGRef& tgt);
  void doLookUpImprovement(AIGTransformer::RefMap& mapToFix);

  bool localCompressByBDD(AIGRef aig, AIGRef& tgt, bool historyLookUp, bool& usedLookUp);
  AIGRef attemptCompressByBDD(AIGRef aig);

  DHMap<AIGRef, size_t> _aigDagSizeCache;

  size_t getAIGDagSize(AIGRef aig);


  typedef const SharedSet<unsigned> USharedSet;
 
  typedef DHMap<AIGRef,USharedSet*> UnsplittableSetMap;
  USharedSet* conjGetUnsplittableSet(USharedSet* p1set, USharedSet* p2set);
  bool tryGetUnsplittableSetLocally(AIGRef a, UnsplittableSetMap& cache, USharedSet*& res, bool doOneUnfolding);
  USharedSet* getUnsplittableSet(AIGRef a, UnsplittableSetMap& cache);



 
  unsigned _maxBDDVarCnt;
  unsigned _reqFactorNum;
  unsigned _reqFactorDenom;


  typedef DHMap<BDDNode*,AIGRef> LookUpMap;
 
  LookUpMap _lookUp;
 
  RefMap _lookUpImprovement;
  bool _lookUpNeedsImprovement;

  AIG& _aig;
  AIGTransformer _atr;
  BDDAIG _ba;

  InterpretedLiteralEvaluator* _ilEval;
};

class AIGCompressingTransformer : public ScanAndApplyFormulaUnitTransformer {
public:
  AIGCompressingTransformer(unsigned maxBddVarCnt=16);

  using ScanAndApplyFormulaUnitTransformer::apply;
  Formula* apply(Formula* f);
  bool applyToDefinition(FormulaUnit* unit, Unit*& res);
  virtual bool apply(FormulaUnit* unit, Unit*& res);

protected:
  virtual void updateModifiedProblem(Problem& prb);

private:
  AIGFormulaSharer _fsh;
  AIGCompressor _compr;
};

}

#endif 
