
/*
 * File AIGInferenceEngine.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __AIGInferenceEngine__
#define __AIGInferenceEngine__

#include "Forwards.hpp"

#include "Lib/DHMultiset.hpp"
#include "Lib/DHSet.hpp"

#include "AIG.hpp"


namespace Shell {

using namespace Lib;

class AIGInferenceEngine
{
public:
  virtual ~AIGInferenceEngine() {}

  void addValidNode(AIGRef node);
  void removeValidNode(AIGRef node);

  virtual AIGRef apply(AIGRef a, AIGStack* premises) = 0;

protected:
  
  
  virtual void addValidNodeImpl(AIGRef node) {}
  virtual void removeValidNodeImpl(AIGRef node) {}

private:
  DHMultiset<AIGRef> _presentNodes;
};

class BottomUpAIGEngine : public AIGInferenceEngine
{
public:
  BottomUpAIGEngine(AIG& aig) : _aig(aig), _atr(aig) {}

  AIGRef apply(AIGRef a, AIGStack* premises);

protected:
  void addPremise(AIGRef prem);
  AIGRef level0Deref(AIGRef a);
  AIGRef level1Deref(AIGRef a);
  virtual AIGRef applyImpl(AIGRef a) = 0;

  AIG& _aig;
private:

  AIGInsideOutPosIterator _buildingIterator;

  DHSet<AIGRef> _currPremiseSet;
  AIGStack* _currPremises;

  AIGTransformer::RefMap _localCache;

  AIGTransformer _atr;
};

class AIGEquivUtils
{
public:
 
  struct Equiv {
    Equiv() {}
    Equiv(AIGRef first_, AIGRef second_);

    vstring toString() const { return "EQ: "+first.toString()+" <=> "+second.toString(); }

    AIGRef first;
    AIGRef second;
  };

  static bool isDisjEquiv(AIGRef a, Equiv& eq);

};

class AIGInliningEngine : public BottomUpAIGEngine
{
public:
  CLASS_NAME(AIGInliningEngine);
  USE_ALLOCATOR(AIGInliningEngine);
  
  AIGInliningEngine(AIG& aig) : BottomUpAIGEngine(aig) {}

protected:
  virtual void addValidNodeImpl(AIGRef node);
  virtual void removeValidNodeImpl(AIGRef node);

  virtual AIGRef applyImpl(AIGRef a);

private:
  void addRewrite(AIGRef src, AIGRef tgt, AIGRef premise);
  void orientEquivalence(AIGRef& lhs, AIGRef& rhs);

  struct RewriteRec {
    RewriteRec() {}
    RewriteRec(AIGRef tgt, AIGRef premise) : tgt(tgt), premise(premise) {}

    AIGRef tgt;
    AIGRef premise;
  };

  struct PremiseRec {
    AIGStack justifiedRewrites;
  };

 
  DHMap<AIGRef,PremiseRec> _premRecs;
  DHMap<AIGRef,RewriteRec> _rewrites;
};

}

#endif 
