
/*
 * File AIGInliner.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __AIGInliner__
#define __AIGInliner__

#include "Forwards.hpp"

#include "Lib/Allocator.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/DHSet.hpp"
#include "Lib/MapToLIFO.hpp"

#include "AIG.hpp"
#include "AIGCompressor.hpp"
#include "AIGRewriter.hpp"

namespace Shell {

using namespace Indexing;


class AIGInliner : public ScanAndApplyFormulaUnitTransformer {

  struct EquivInfo {
    EquivInfo(Literal* lhs, Formula* rhs, FormulaUnit* unit);

    
    Literal* lhs;
    Formula* rhs;
    FormulaUnit* unit;
    Literal* posLhs;
    bool active;

    
    AIGRef activeAIGRhs;













    CLASS_NAME(AIGInliner::EquivInfo);
    USE_ALLOCATOR(EquivInfo);

    static bool litIsLess(Literal* l1, Literal* l2);
    static EquivInfo* tryGetEquiv(FormulaUnit* fu);
  };

  
  bool _onlySingleAtomPreds;

  Stack<AIGRef> _relevantAigs;

  Stack<EquivInfo*> _eqInfos;

  LiteralSubstitutionTree* _lhsIdx;
  DHMap<Literal*, EquivInfo*> _defs;
  DHMap<FormulaUnit*, EquivInfo*> _unit2def;

public:
  typedef AIGRewriter::PremiseSet PremSet;
  typedef AIGRewriter::PremRef PremRef;
  typedef AIGRewriter::RefMap PremRefMap;
  typedef DHMap<AIGRef,AIGRef> RefMap;
private:
  PremRefMap _inlMap;
  RefMap _simplMap;

#if VDEBUG
  
  DHSet<AIGRef> _relevantAigsSet;
#endif

  mutable AIGFormulaSharer _fsh;
  AIG& _aig;
  AIGRewriter _atr;
  AIGCompressor _acompr;



  bool addInfo(EquivInfo* inf);
  void collectDefinitionsAndRelevant(UnitList* units);
  bool tryExpandAtom(AIGRef atom, PremRef& res);


public:
  AIGInliner();
  ~AIGInliner();

  AIG& aig() const { return _aig; }
  AIGFormulaSharer& fsh() const { return _fsh; }

  using ScanAndApplyFormulaUnitTransformer::apply;

  void scan(UnitList* units);
  virtual bool apply(FormulaUnit* unit, Unit*& res);

  void addRelevant(AIGRef a);
  void addRelevant(Formula* f);
  AIGRef apply(AIGRef a, PremSet*& prems);
  Formula* apply(Formula* f, PremSet*& prems);

protected:
  virtual void updateModifiedProblem(Problem& prb);
};



}

#endif 
