
/*
 * File ConstraintReaderBack.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __ConstraintReader__
#define __ConstraintReader__
#if GNUMP
#include "Forwards.hpp"

#include "Kernel/Constraint.hpp"
#include "MPSLib/Model.h"

#include "SMTPAR.hpp"

namespace Shell {

using namespace Kernel;

class ConstraintReader {
public:
  ConstraintReader(SMTParser& parser);
  
  ConstraintRCList* constraints();
private:
  typedef Stack<Constraint::Coeff> CoeffStack;

  void readCoefs(SMTParser::Term* term, CoeffStack& coeffs, CoeffNumber& freeCoeff);

  static CoeffNumber readNumber(SMTParser::Term* term);

  Signature& _sig;
  SMTParser& _parser; 
};

class SMTConstraintReader{
public:
  SMTConstraintReader(Parse::SMTLIB& parser);
  ConstraintRCList* constraints();
private:

  Parse::SMTLIB& _parser;
};

class SMTLib2ConstraintReader{
public:
  SMTLib2ConstraintReader(Parse::SMTLIB2& parser);
  
  ConstraintRCList* constraints();

private:

  typedef Stack<Constraint::Coeff> CoefficientStack;
  void readCoefficients(Term* term, CoefficientStack& coeffs, CoeffNumber& freeCoeff, CoeffNumber val=CoeffNumber(1));

  ConstraintRCPtr getConstraint(Literal* literal);
  CoeffNumber readNumber(Term* term);
  bool isNumber(vstring term);
  Signature& _sig;
  Parse::SMTLIB2& _parser;
};

class MpsConstraintReader
{

public:
    MpsConstraintReader(Model& model);
    virtual ~MpsConstraintReader();
    ConstraintRCList* constraints();

private: 
   typedef Stack<Constraint::Coeff> CoeffStack;
   Signature& _sig;
   Model& _model; 
   
};
}
#endif 
#endif 


