
/*
 * File DistinctGroupExpansion.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#include "Lib/Environment.hpp"
#include "Lib/Stack.hpp"
#include "Lib/List.hpp"

#include "Kernel/Signature.hpp"
#include "Kernel/Problem.hpp"
#include "Kernel/Unit.hpp"
#include "Kernel/Term.hpp"
#include "Kernel/Formula.hpp"
#include "Kernel/FormulaUnit.hpp"
#include "Kernel/Inference.hpp"
#include "Kernel/SortHelper.hpp"
#include "Kernel/Connective.hpp"

#include "Options.hpp"
#include "DistinctGroupExpansion.hpp"

using namespace Shell;

void DistinctGroupExpansion::apply(Problem& prb)
{
  CALL("DistinctGroupExpansion::apply(Problem&)");

  if(apply(prb.units())){
    prb.invalidateProperty();
    prb.refortFormulasAdded();
    prb.reportEqualityAdded(false); 
  }

}

bool DistinctGroupExpansion::apply(UnitList*& units)
{
  CALL("DistinctGroupExpansion::apply(UnitList*&)");

  bool added=false;

  Stack<Stack<unsigned>*> group_members = env.signature->getDistinctGroupMembers();

  bool someLeft = false;

  for(unsigned i=0;i<group_members.size();i++){
    Stack<unsigned>* members = group_members[i];
    if(members->size() > 0){
 
      
      
      
      if(i>0 && i<=3) someLeft=true;

      
      
      if( members->size()>1 && ((members->size() < 5 || env.options->bfnt()))){
        added=true;
        Formula* expansion = expand(*members);
        if(env.options->showPreprocessing()){
          env.out() << "  expansion adding " << expansion->toString() << endl;
        }
        
        UnitList::push(new FormulaUnit(expansion, new Inference(Inference::INPUT),Unit::AXIOM),units);
      }
      else someLeft=true;
    }
  } 

  if(!someLeft){ env.signature->noDistinctGroupsLeft(); }

  return added;
}

Formula* DistinctGroupExpansion::expand(Stack<unsigned> constants)
{
  CALL("DistinctGroupExpansion::expand");

  ASS(constants.size()>=2);
  
  if(constants.size()==2){
    TermList a = TermList(Term::createConstant(constants[0]));
    TermList b = TermList(Term::createConstant(constants[1]));
    unsigned sort = SortHelper::getResultSort(a.term());
    return new AtomicFormula(Literal::createEquality(false,a,b,sort));
  }

  
  FormulaList* diseqs = 0; 

  for(unsigned i=0;i<constants.size();i++){
    TermList a = TermList(Term::createConstant(constants[i]));
    ASS(a.isSafe());
    unsigned sort = SortHelper::getResultSort(a.term());

    for(unsigned j=0;j<i;j++){
      TermList b = TermList(Term::createConstant(constants[j]));
      ASS(b.isSafe());
      
      Formula* new_dis = new AtomicFormula(Literal::createEquality(false,a,b,sort));
      if(diseqs) FormulaList::push(new_dis,diseqs);
      else diseqs = new FormulaList(new_dis);

    }
  }

  
  return new JunctionFormula(Connective::AND, diseqs);

}

