
/*
 * File DistinctProcessor.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#include "Lib/Environment.hpp"

#include "Kernel/Formula.hpp"
#include "Kernel/FormulaUnit.hpp"
#include "Kernel/Signature.hpp"
#include "Kernel/Unit.hpp"

#include "DistinctProcessor.hpp"

namespace Shell
{



bool DistinctProcessor::isDistinctPred(Literal* l)
{
  CALL("DistinctProcessor::isDistinctPred");

  
  
  

  
  
  
  const char* n = l->predicateName().c_str();
  return n[0]=='$' && memcmp(n+1,"distinct",8)==0;
}

bool DistinctProcessor::apply(FormulaUnit* unit, Unit*& res)
{
  CALL("DistinctProcessor::apply");

  Formula* form = unit->formula();


  static Stack<unsigned> distConsts;
  if(form->connective()==LITERAL) {
    Literal* tlLit = form->literal();
    if(isDistinctPred(tlLit) && tlLit->isPositive()) {
      distConsts.reset();
      bool justConsts = true;
      Literal::Iterator argIt(tlLit);
      while(argIt.hasNext()) {
	TermList a = argIt.next();
	if(!a.isTerm() || a.term()->arity()!=0) {
	  justConsts = false;
	  break;
	}
	distConsts.push(a.term()->functor());
      }

      if(justConsts) {
	unsigned grpIdx = env.signature->createDistinctGroup(unit);
	while(distConsts.isNonEmpty()) {
	  env.signature->addToDistinctGroup(distConsts.pop(), grpIdx);
	}
      }else{
        USER_ERROR("$distinct should only be used positively with constants");
      }
    }
  }

  return false;
}

bool DistinctProcessor::apply(Clause* cl, Unit*& res) {
  return false;
}

}
