
/*
 * File EquivalentVariableRemover.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __EquivalentVariableRemover__
#define __EquivalentVariableRemover__
#if GNUMP

#include <utility>

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/IntUnionFind.hpp"

namespace Shell {

using namespace std;
using namespace Lib;
using namespace Kernel;

class EquivalentVariableRemover {
public:
  EquivalentVariableRemover();

  bool apply(ConstraintRCList*& lst);
private:
  void reset();
  void scan(ConstraintRCList* lst);

  Var getRoot(Var v) { return _eqClasses.root(v); }
  bool remainsSame(Var v) { return getRoot(v)==v; }
  bool remainsSame(Constraint& c);

  struct VarMapper;

  typedef pair<Var,Var> VarPair;
  enum PairStatus {
    FIRST_POS = 1,
    FIRST_NEG = -1,
    BOTH = 0
  };
  DHMap<VarPair, PairStatus> _pairs;
  IntUnionFind _eqClasses;
  bool _haveEquivalences;
};

}
#endif 
#endif 
