
/*
 * File LaTeX.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __LaTeX__
#define __LaTeX__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/Stack.hpp"
#include "Lib/VString.hpp"

#include "Kernel/Connective.hpp"
#include "Kernel/InferenceStore.hpp"











namespace Shell {

using namespace Kernel;

class LaTeX
{
public:
  LaTeX() : _nextNodeNum(0) {}

  vstring header();
  vstring footer();
  vstring refutationToString(Unit* ref);



  vstring toString(const Term&) const;
  vstring toString(const vstring& funOrPred,const TermList& args) const;
  vstring toString(Unit*);
private:




  vstring varToString(unsigned num) const;
  vstring toString(TermList*,bool single=false) const;
  vstring toString(Literal*) const;
  vstring toString(Clause*);
  vstring toString(Formula*) const;
  vstring toString(Formula*, Connective c) const;

  vstring getClauseLatexId(UnitSpec cs);

  
  vstring toStringAsInference(Unit*);
  vstring toStringAsInference(UnitSpec cs, InferenceStore::FullInference* inf);

  vstring symbolToString (unsigned num, bool pred) const;


  vstring toString(BDDNode*);
  vstring getBDDVarName(int var);

  Stack<vstring> definitionStack;
  int _nextNodeNum;
  DHMap<BDDNode*,vstring> _nodeNames;
}; 


}

#endif 

