
/*
 * File Lexer.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __Lexer__
#define __Lexer__

#include <iostream>

#include "Lib/Array.hpp"
#include "Lib/Exception.hpp"

#include "Token.hpp"

using namespace std;
using namespace Lib;


namespace Shell {

class Lexer;

class LexerException 
  : public Exception
{
 public:                                
  LexerException(vstring message,const Lexer&);
  void cry(ostream&);
  ~LexerException() {}
 protected:
  vstring _message;
}; 


class Lexer 
{
public:
  Lexer(istream& in);
 
  bool isAtEndOfFile () const { return _eof; }
 
  int lastCharacter () const { return _lastCharacter; }
 
  int lineNumber () const { return _lineNumber; }
  virtual void readToken (Token&) = 0;
  virtual ~Lexer () {}
  int lookAhead();

protected:
 
  int  _lastCharacter;
 
  Array<char> _charBuffer;
 
  int _charCursor;
 
  std::istream& _stream;
 
  bool _eof;
 
  int _lineNumber;
 
  int _lookAheadChar;

  bool readNextChar();
  void readNumber(Token&);
  void readUnsignedInteger();
  void saveLastChar();
  void saveChar(int character);
  void saveTokenText(Token&);
 
  static bool isDigit(int charCode) 
  { return charCode >= '0' && charCode <= '9'; }
 
  static bool isLetter(int charCode) 
  { return (charCode >= 'A' && charCode <= 'Z') || 
           (charCode >= 'a' && charCode <= 'z'); }
  void readSequence(const char* chars);
}; 

}

#endif

