
/*
 * File Naming.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __Naming__
#define __Naming__

#include "Kernel/Formula.hpp"
#include "Kernel/Unit.hpp"

using namespace Kernel;

namespace Shell {

class Naming
{
public:
  Naming (int threshold, bool preserveEpr);
  FormulaUnit* apply(FormulaUnit* unit,UnitList*& defs);
private:
 
  enum Where {
   
    ON_TOP,
   
    UNDER_IFF,
   
    OTHER
  };

 
  enum ApplyWhat {
    APPLY_SUB_TOP,
    APPLY_SUB_AND,
    APPLY_SUB_OR,
    APPLY_SUB_IFFXOR,
    APPLY_SUB_FORALLEXISTS,
    APPLY_LIST_TOP,
    APPLY_LIST_POST
  };

 
  struct ResultSub {
    int pos;
    int neg;
    Formula* res;
  };

 
  struct ResultList {
    FormulaList* res;
  };

 
  union Result {
    ResultSub  resSub;
    ResultList resList;
  };

 
  struct SubtaskAndOr {
    int* cls;
    int* negCls;
  };

 
  struct SubtaskForallExists {
    bool varFlagSet;
  };

 
  struct TaskApplySub {
    Formula* f;
    Where where;
    union {
      SubtaskAndOr taskAndOr;
      SubtaskForallExists taskForallExists;
    };
  };

 
  struct TaskApplyList {
    FormulaList* fs;
    Where where;
    int* results;
    int* negResults;
  };

 
  struct Task {
    ApplyWhat fncTag;  
    union {
      TaskApplySub  taskApplySub;
      TaskApplyList taskApplyList;
    };
  };

 
  int _threshold;
 
  bool _preserveEpr;

 
  bool _varsInScope;

  bool canBeInDefinition(Formula* f,Where where);

 
  UnitList* _defs;

 
  Formula* apply_iter(Formula* top_f);

  Formula* apply_sub(Formula* subformula,Where where,int& pos,int& neg);
  FormulaList* apply_list(FormulaList* subformulas,
		     Where where,
		     int* results,
		     int* resultsNeg);
  Formula* introduceDefinition(Formula* f,bool iff);

  Literal* getDefinitionLiteral(Formula* f, Formula::VarList* freeVars);
}; 

}

#endif
