
/*
 * File PARSER_TKV.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __Parser__
#define __Parser__

#include "Lib/DHMap.hpp"
#include "Lib/IntNameTable.hpp"
#include "Lib/Exception.hpp"
#include "Lib/XML.hpp"

#include "Token.hpp"

using namespace std;
using namespace Lib;
using namespace Kernel;

#define TOKEN_BUFFER_SIZE 3

namespace Shell {

class Lexer;

class ParserException 
  : public Exception
{
 public:                                
  ParserException (vstring message,const Token&);
  void cry (ostream&);
  XMLElement toXML () const;
  ~ParserException () {}
 protected:
  vstring _message;
}; 

class Parser 
{
public:
  Parser(Lexer& lexer);
protected:
 
  Lexer& _lexer;
 
  Token _tokens[TOKEN_BUFFER_SIZE];
 
  int _tokenCursor;
 
  int _noOfTokens;
 
  IntNameTable _vars;

  void consumeToken();
  void consumeToken(TokenType);
  void expectToken(TokenType);
  void expectKeyword(const char* keyword);
  void readToken();
  void readToken(TokenType);
  Token& lookAhead (int lookahead);
  Token& currentToken();
  void consumeToken(TokenType,vstring errorMessage);
  void expectToken(TokenType,vstring errorMessage);
  void readToken(TokenType,vstring errorMessage);
  void terminate(vstring errorMessage) __attribute__((noreturn));
 
  inline
  Token& currentToken1()
  {
    if (_noOfTokens == 0) {
      readToken();
    }

    return currentToken();
  } 
  int var(const Token& t);
  static double real(const Token& t);
  static int integer(const Token& t);
  static long long unsigned unsigned64(const Token& t);
}; 

}

#endif

