
/*
 * File PDInlinerBetter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __PDInliner__
#define __PDInliner__

#include "Forwards.hpp"

#include "Lib/Array.hpp"
#include "Lib/DArray.hpp"
#include "Lib/MultiCounter.hpp"
#include "Lib/SharedSet.hpp"
#include "Lib/Set.hpp"
#include "Lib/Stack.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class PDInliner {
public:
  PDInliner(bool axiomsOnly=false, bool nonGrowing=false);
  ~PDInliner();

  void apply(Problem& prb);

 
  bool apply(UnitList*& units, bool inlineOnlyEquivalences=false);
  Unit* apply(Unit* u);
  Unit* apply(Clause* u);
  FormulaUnit* apply(FormulaUnit* u);

 
  bool tryGetDef(FormulaUnit* unit);
  bool tryGetPredicateEquivalence(FormulaUnit* unit);
  bool scanAndRemoveDefinitions(UnitList*& units, bool equivalencesOnly);

  void prepareScannedDefs();

  bool addAsymetricDefinition(Literal* lhs, Formula* posBody, Formula* negBody, Formula* dblBody,
      FormulaUnit* premise=0);

  void updatePredOccCounts(Unit* u);
  bool isNonGrowingDef(Literal* lhs, Formula* rhs);

 
  bool tryGetDef(FormulaUnit* unit, Literal* lhs, Formula* rhs);
private:


  typedef SharedSet<unsigned> DepSet;

  friend class EPRRestoring;
  friend class EPRInlining;

  struct Applicator;
  struct DefDep;
  struct PDef;

  struct InliningState
  {
    Stack<Unit*> premises;
    bool needsRectify;
    bool needsConstantSimplification;

    void reset()
    {
      CALL("PDInliner::InliningState::reset");
      premises.reset();
      needsRectify = false;
      needsConstantSimplification = false;
    }
  };

  Formula* apply(int polarity, Formula* form, InliningState& state);
  void getInferenceAndInputType(Unit* transformedUnit, InliningState& state, Inference*& inf, Unit::InputType& inp);


  bool isEligible(FormulaUnit* u);


  static bool isDefinitionHead(Literal* l);

  ZIArray<PDef*> _defs;

  ZIArray<DefDep*> _deps;
 
  DArray<Stack<unsigned> > _dependent;

 
  MultiCounter _predOccCounts;

  bool _axiomsOnly;
  bool _nonGrowing;
};

}

#endif 
