
/*
 * File PredicateIndexIntroducer.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __PredicateIndexIntroducer__
#define __PredicateIndexIntroducer__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/DHSet.hpp"
#include "Lib/SharedSet.hpp"

#include "Kernel/FormulaTransformer.hpp"



namespace Shell {

using namespace Lib;
using namespace Kernel;

class PredicateIndexIntroducer : public ScanAndApplyLiteralTransformer {
public:
  PredicateIndexIntroducer();

  virtual void scan(UnitList* units);

  using ScanAndApplyLiteralTransformer::apply;

protected:
  virtual Literal* apply(Literal* lit, UnitStack& premAcc);

  virtual void updateModifiedProblem(Problem& prb);
private:
  

  typedef const SharedSet<unsigned> DistGrpSet;

  struct ArgOccInfo {
    ArgOccInfo() : _new(true) {}

    void scanArg(PredicateIndexIntroducer& parent, TermList t);

    bool _new;
    bool _eligible;
    DistGrpSet* _distGrps;
    Term* _onlyOcc;
  };

  void scan(Literal* lit);
  DistGrpSet* getDistGrps(TermList trm);

 
  bool _assumeSSDistinctGroup;
  unsigned _ssDistinctGroupID;

  DHSet<Literal*> _seen;
 
  typedef pair<unsigned,unsigned> ArgId;
  typedef DHMap<ArgId,ArgOccInfo> ArgOccInfoMap;
  ArgOccInfoMap _predArgOccInfos;

  typedef DHMap<unsigned,Stack<unsigned> > IdxPredArgsMap;
  IdxPredArgsMap _indexedPredArgs;


private:
  

  unsigned getIndexedPred(Literal* lit);

 
  typedef pair<unsigned,TermStack> IdxPredKey;
  DHMap<IdxPredKey,unsigned> _idxPreds;
};

}

#endif 
