
/*
 * File Preprocess.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __Preprocess__
#define __Preprocess__

#include "Kernel/Unit.hpp"
#include "Forwards.hpp"

namespace Shell {

using namespace Kernel;

class Property;
class Options;

class Preprocess
{
public:
 
  explicit Preprocess(const Options& options)
  : _options(options),
    _clausify(true)
  {}
  void preprocess(Problem& prb);
#if GNUMP
  void preprocess(ConstraintRCList*& constraints);
#endif

  void preprocess1(Problem& prb);
 
  void turnClausifierOff() {_clausify = false;}
private:
  void preprocess2(Problem& prb);
  void naming(Problem& prb);
  void secondStageEprPreservingNaming(Problem& prb);
  Unit* preprocess3(Unit* u);
  void preprocess3(Problem& prb);
  void clausify(Problem& prb);

 
  const Options& _options;
 
  bool _clausify;
#if GNUMP
  void unfoldEqualities(ConstraintRCList*& constraints);
#endif
}; 


}

#endif
