
/*
 * File Refutation.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#include "Debug/Tracer.hpp"

#include "Lib/Hash.hpp"
#include "Lib/Set.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/Clause.hpp"
#include "Kernel/Inference.hpp"

#include "Refutation.hpp"

using namespace Lib;
using namespace Kernel;
using namespace Shell;

Refutation::Refutation(Unit* unit,bool detailed)
  : _goal(unit)
{
}

void Refutation::output(ostream& str)
{
  CALL("Refutation::output");

  Stack<Unit*> units(128);
  Set<Unit*,Refutation> done;

  units.push(_goal);
  while (! units.isEmpty()) {
    Unit* unit = units.pop();
    if (done.contains(unit)) {
      continue;
    }
    done.insert(unit);
    str << unit->toString() << "\n";
    Inference* inf = unit->inference();
    Inference::Iterator it = inf->iterator();
    while (inf->hasNext(it)) {
      units.push(inf->next(it));
    }
  }
} 

unsigned Refutation::hash (Unit* unit)
{
  return Hash::hash(reinterpret_cast<void*>(unit));
}
