
/*
 * File SMTPAR.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __SMTParser__
#define __SMTParser__

#include <iostream>

#include "Lib/VString.hpp"
#include "../Lib/Exception.hpp"
#include "PARSER_TKV.hpp"

using namespace std;

namespace Shell {

class SMTLexer;

class SMTParser
  : public Parser
{
public:
 
  enum Logic {
   


    QF_LRA

  };
 
  enum Status {
   
    SAT,
   
    UNSAT,
   
    UNKNOWN
  };

 
  enum Sort {
   
    REAL
  };

 
  struct Annotation {
   
    vstring attribute;
   
    vstring value;
   
    Annotation* next;
  }; 

 
  struct FunctionDeclaration {
    FunctionDeclaration(const vstring& name);
   
    vstring name;
   
    Sort sort;
   
    Annotation* annotations;
   
    FunctionDeclaration* next;
  }; 

 
  struct PredicateDeclaration {
    PredicateDeclaration(const vstring& name);
   
    vstring name;
   
    Sort sort;
   
    Annotation* annotations;
   
    PredicateDeclaration* next;
  }; 

 
  enum Connective {
   
    ATOM,
   
    NOT,
   
    AND,
   
    OR,
   
    IMPLY,
   
    IFF,
   
    XOR,
   
    FORALL,
   
    EXISTS
  };

 
  enum TermType {
    TERM_INT,
    TERM_REAL,
    TERM_ARITH,
    TERM_COMPLEX
  }; 

 
  struct Term {
   
    TermType kind;
   
    vstring fun;
   
    Term* args;
   
    Term* next;
   
   
    Annotation* annotations;
    Term(TermType tt,vstring str)
      : kind(tt),
	fun(str),
	args(0),
	next(0),
	annotations(0)
    {};

    void toString (){
      cout<<this->fun<<" ";
        if (this->args != 0){
          this->args->toString();
        };
        if (this->next != 0){
          this->next->toString();
        };
    };

  };

 
  struct Atom {
   
    Atom(vstring nm)
      : pred(nm),
	args(0),
	annotations(0)
    {}
   
    vstring pred;
   
    Term* args;
   
    Annotation* annotations;
    void toString (){
      this->args->toString();
    };


  };




 
  struct Formula {
   
    Formula(Connective con)
      : connective(con),
	args(0),
	next(0)
    {}
   
    Connective connective;
    union {
     
      Formula* args;
     
      Atom* atom;
    };
   
    Formula* next;
  };

 
  struct Benchmark {
    Benchmark(const vstring& nm);
   
    vstring name;
   
    Logic logic;
   
    Status status;
   
    FunctionDeclaration* functionDeclarations;
   
    PredicateDeclaration* predicateDeclarations;
   
    Annotation* annotations;
   
    Formula* formula;
  };

  explicit SMTParser(SMTLexer& lexer);
  Benchmark* benchmark();
  Annotation* annotation();
private:
  void predicates(PredicateDeclaration**);
  void functions(FunctionDeclaration**);
  void formula(Formula**);
  void formulas(Formula**);
  void term(Term**);
  void terms(Term**);

}; 

}

#endif

