
/*
 * File SymCounter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __sym_counter__
#define __sym_counter__

#include "Kernel/Unit.hpp"

namespace Kernel {
  class Signature;
  class Formula;
  class Clause;
  class Term;
  class TermList;
  class Literal;
}

using namespace Kernel;

namespace Shell {

class SymCounter
{
 public:

  class Fun {
    int _occ;
   public:
    Fun () : _occ(0) {}
    int occ () const { return _occ; }
    void add (int add) { _occ += add; }
  };

  class Pred {
    int _pocc;  
    int _nocc;  
    int _docc;  
   public:
    Pred ()
      : _pocc (0),
        _nocc (0),
        _docc (0)
        {}
    int pocc () const { return _pocc; }
    int nocc () const { return _nocc; }
    int docc () const { return _docc; }
    void add (int polarity, int add);
  };

  explicit SymCounter (Signature&);
  ~SymCounter ();

  
  void count(const UnitList*, int);
  void count(const Formula*, int);
  void count(const Clause*, int);
  void count(const Formula*, int polarity, int add);
  void count(const Literal*, int polarity, int add);
  void count(const TermList* t, int add);

 
  Pred& getPred (int n)
  {
    ASS(n < _noOfPreds);
    return _preds[n];
  }
 
  Fun& getFun (int n)
  {
    ASS(n < _noOfFuns);
    return _funs[n];
  }

 private:
  
 
  int _noOfPreds;
 
  int _noOfFuns;
 
  Pred* _preds;
 
  Fun* _funs;
}; 

}

#endif 
