
/*
 * File TheoryFinder.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __TheoryFinder__
#define __TheoryFinder__

#include "Kernel/Unit.hpp"

namespace Kernel {
  class Clause;
  class Formula;
  class Literal;
}

using namespace Kernel;

namespace Shell {

class Property;

class TheoryFinder {
public:
  TheoryFinder(const UnitList*,Property* property);
  ~TheoryFinder();
  int search();
  static bool matchCode(const void* obj,const unsigned char* code);
  static bool matchKnownExtensionality(const Clause*);

private:
  bool matchAll(const Unit* unit);
  bool matchAll(const Clause* clause);
  bool matchAll(const Formula* formula);
  bool matchAll(const Literal* literal);
  bool matchCode(const void* obj,const unsigned char* code,unsigned long prop);







  bool matchC(const Literal*);
  bool matchA(const Literal*);
  bool matchLeftInverse(const Literal*);
  bool matchLeftIdentity(const Literal*);
  bool matchRightInverse(const Literal*);
  bool matchRightIdentity(const Literal*);
  bool matchLeftDistributivity(const Literal*);
  bool matchRightDistributivity(const Literal*);
  bool matchRobbins(const Literal*);
  bool matchIdempotence(const Literal*);
  bool matchAssociator(const Literal*);
  bool matchCommutator(const Literal*);
  bool matchAlternative(const Literal*);
  bool matchAbsorption(const Literal*);
  bool matchCombinatorS(const Literal*);
  bool matchCombinatorB(const Literal*);
  bool matchCombinatorT(const Literal*);
  bool matchCombinatorO(const Literal*);
  bool matchCombinatorQ(const Literal*);

  bool matchExtensionality(const Formula*);
  bool matchExtensionality(const Clause*);
  bool matchCondensedDetachment1(const Clause*);
  bool matchCondensedDetachment2(const Clause*);
  bool matchFLD1(const Clause*);
  bool matchFLD2(const Clause*);
  bool matchSubset(const Formula*);
  bool matchSubset(const Clause*);
  bool matchListConstructors(const Formula*);
  bool matchTest(const Formula*);







 
  enum Code {
   
    NEWVAR,
   
    OLDVAR,
   
    NEWFUN,
   
    OLDFUN,
   
    CAND,
   
    COR,
   
    CIMP,
   
    CNOT,
   
    CIFF,
   
    CXOR,
   
    CFORALL,
   
    CEXISTS,
   
    EQL,
   
    POS,
   
    NEG,
   
    TERM,
   
    FORM,
   
    NEWFUN1,
   
    OLDFUN1,
   
    OLDVAR1,
   
    PLIT,
   
    NLIT,
   
    CLS,
   
    NEWPRED,
   
    OLDPRED,
   
    END
  };

  class Backtrack;

 
  const UnitList* _units;
 
  Property* _property;  
};

}

#endif
