
/*
 * File UIHelper.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __UIHelper__
#define __UIHelper__

#include <ostream>

#include "Forwards.hpp"
#include "Options.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class UIHelper {
public:
  static Problem* getInputProblem(const Options& opts);
  static void outputResult(ostream& out);

 
  static bool haveConjecture() { return s_haveConjecture; }
  static void setConjecturePresence(bool haveConjecture) { s_haveConjecture=haveConjecture; }

  static void outputAllPremises(ostream& out, UnitList* units, vstring prefix="");


  static void outputSatisfiableResult(ostream& out);
  static void outputSaturatedSet(ostream& out, UnitIterator uit);

  static void outputSymbolDeclarations(ostream& out);
  static void outputSymbolTypeDeclarationIfNeeded(ostream& out, bool function, unsigned symNumber);

 
  static bool cascMode;
 
  static bool cascModeChild;

 
  static bool satisfiableStatusWasAlreadyOutput;
#if GNUMP
  static ConstraintRCList* getInputConstraints(const Options& opts);
  static ConstraintRCList* getPreprocessedConstraints(const ConstraintRCList* inputConstraints);
  static ConstraintRCList* getPreprocessedConstraints(const Options& opts);
  
  static void outputConstraint(const Constraint& constraint, ostream& out, Options::InputSyntax syntax = Options::IS_HUMAN);
  static void outputConstraints(ConstraintList* constraints, ostream& out, Options::InputSyntax syntax=Options::IS_HUMAN);
  
  static void outputAssignment(Assignment& assignemt, ostream& out, Options::InputSyntax syntax=Options::IS_HUMAN);
#endif 
  
private:
#if GNUMP
  static void outputConstraintInHumanFormat(const Constraint& constraint, ostream& out);
  static void outputConstraintInSMTFormat(const Constraint& constraint, ostream& out);
#endif 

  static bool unitSpecNumberComparator(UnitSpec us1, UnitSpec us2);
  static void addCommentIfCASC(ostream&); 

  static bool s_haveConjecture;
#if VDEBUG
  static bool _inputHasBeenRead;
#endif
  
};

}

#endif 
