
/*
 * File TabulationContainers.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */


#ifndef __TabulationContainers__
#define __TabulationContainers__

#include "Forwards.hpp"

#include "Lib/MapToLIFO.hpp"
#include "Lib/ScopedPtr.hpp"

#include "Indexing/LiteralIndex.hpp"
#include "Indexing/LiteralIndexingStructure.hpp"


#include "Saturation/ClauseContainer.hpp"
#include "Saturation/AWPassiveClauseContainer.hpp"

namespace Tabulation {

using namespace Lib;
using namespace Kernel;
using namespace Indexing;
using namespace Inferences;
using namespace Saturation;

class GIContainer : public ClauseContainer {
public:
  GIContainer();

  virtual void add(Clause* c);

  GeneratingLiteralIndex* getIndex() { return _unifIndex.ptr(); }
private:
  ScopedPtr<GeneratingLiteralIndex> _unifIndex;
};

typedef AWClauseContainer GoalContainer;

class GoalLiteralContainer
{
public:
  GoalLiteralContainer();

  void add(Literal* lit);
  bool isSubsumed(Literal* lit);
private:
  ScopedPtr<LiteralIndexingStructure> _index;
};

class GoalProducer
{
public:
  GoalProducer(TabulationAlgorithm& alg);

  void addRule(Clause* goal, Literal* activator);

  void onLemma(Clause* lemma);

  LiteralIndexingStructure& getLemmaIndex() { return *_lemmaIndex; }

  static Clause* makeInstance(Clause* resCl, ResultSubstitution& subst, bool clIsResult);
private:


  ScopedPtr<LiteralIndexingStructure> _lemmaIndex;
  ScopedPtr<LiteralIndexingStructure> _activatorIndex;

  TabulationAlgorithm& _alg;
};

}

#endif 
